# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/postgresql/v1alpha1/template.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.msp.v1alpha1.resource import template_pb2 as nebius_dot_msp_dot_v1alpha1_dot_resource_dot_template__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-nebius/msp/postgresql/v1alpha1/template.proto\x12\x1enebius.msp.postgresql.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a+nebius/msp/v1alpha1/resource/template.proto\"\xeb\x01\n\x0cTemplateSpec\x12Q\n\tresources\x18\x01 \x01(\x0b\x32+.nebius.msp.v1alpha1.resource.ResourcesSpecB\x06\xbaH\x03\xc8\x01\x01R\tresources\x12\x44\n\x05hosts\x18\x02 \x01(\x0b\x32&.nebius.msp.v1alpha1.resource.HostSpecB\x06\xbaH\x03\xc8\x01\x01R\x05hosts\x12\x42\n\x04\x64isk\x18\x03 \x01(\x0b\x32&.nebius.msp.v1alpha1.resource.DiskSpecB\x06\xbaH\x03\xc8\x01\x01R\x04\x64iskBv\n%ai.nebius.pub.msp.postgresql.v1alpha1B\rTemplateProtoP\x01Z<github.com/nebius/gosdk/proto/nebius/msp/postgresql/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.postgresql.v1alpha1.template_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%ai.nebius.pub.msp.postgresql.v1alpha1B\rTemplateProtoP\001Z<github.com/nebius/gosdk/proto/nebius/msp/postgresql/v1alpha1'
  _TEMPLATESPEC.fields_by_name['resources']._options = None
  _TEMPLATESPEC.fields_by_name['resources']._serialized_options = b'\272H\003\310\001\001'
  _TEMPLATESPEC.fields_by_name['hosts']._options = None
  _TEMPLATESPEC.fields_by_name['hosts']._serialized_options = b'\272H\003\310\001\001'
  _TEMPLATESPEC.fields_by_name['disk']._options = None
  _TEMPLATESPEC.fields_by_name['disk']._serialized_options = b'\272H\003\310\001\001'
  _globals['_TEMPLATESPEC']._serialized_start=156
  _globals['_TEMPLATESPEC']._serialized_end=391
# @@protoc_insertion_point(module_scope)



from collections.abc import Sequence
from typing import Any, TypeVar

from grpc.aio._typing import ChannelArgumentType

T = TypeVar("T")


class WrongTypeError(Exception):
    def __init__(self, name: str, exp_type: type[T], received: Any) -> None:
        super().__init__(
            f"Option with name {name} expected type is {type(exp_type)},"
            f" found {type(received)}"
        )


def pop_option(
    args: ChannelArgumentType,
    name: str,
    expected_type: type[T],
) -> tuple[ChannelArgumentType, T | None]:
    ret, found = pop_options(args, name, expected_type)
    return ret, found[-1] if len(found) > 0 else None


def pop_options(
    args: ChannelArgumentType,
    name: str,
    expected_type: type[T],
) -> tuple[ChannelArgumentType, Sequence[T]]:
    ret = list[tuple[str, Any]]()
    found = list[T]()
    for arg in args:
        if arg[0] == name:
            if isinstance(arg[1], expected_type):
                found.append(arg[1])
            else:
                WrongTypeError(name, expected_type, arg[1])
        else:
            ret.append(arg)
    return ret, found


INSECURE = "nebius.insecure"
COMPRESSION = "nebius.compression"



from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class ExchangeTokenRequest(_message.Message):
    __slots__ = ["grant_type", "requested_token_type", "subject_token", "subject_token_type", "scopes", "audience", "actor_token", "actor_token_type", "resource"]
    GRANT_TYPE_FIELD_NUMBER: _ClassVar[int]
    REQUESTED_TOKEN_TYPE_FIELD_NUMBER: _ClassVar[int]
    SUBJECT_TOKEN_FIELD_NUMBER: _ClassVar[int]
    SUBJECT_TOKEN_TYPE_FIELD_NUMBER: _ClassVar[int]
    SCOPES_FIELD_NUMBER: _ClassVar[int]
    AUDIENCE_FIELD_NUMBER: _ClassVar[int]
    ACTOR_TOKEN_FIELD_NUMBER: _ClassVar[int]
    ACTOR_TOKEN_TYPE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_FIELD_NUMBER: _ClassVar[int]
    grant_type: str
    requested_token_type: str
    subject_token: str
    subject_token_type: str
    scopes: _containers.RepeatedScalarFieldContainer[str]
    audience: str
    actor_token: str
    actor_token_type: str
    resource: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, grant_type: _Optional[str] = ..., requested_token_type: _Optional[str] = ..., subject_token: _Optional[str] = ..., subject_token_type: _Optional[str] = ..., scopes: _Optional[_Iterable[str]] = ..., audience: _Optional[str] = ..., actor_token: _Optional[str] = ..., actor_token_type: _Optional[str] = ..., resource: _Optional[_Iterable[str]] = ...) -> None: ...

class CreateTokenResponse(_message.Message):
    __slots__ = ["access_token", "issued_token_type", "token_type", "expires_in", "scopes"]
    ACCESS_TOKEN_FIELD_NUMBER: _ClassVar[int]
    ISSUED_TOKEN_TYPE_FIELD_NUMBER: _ClassVar[int]
    TOKEN_TYPE_FIELD_NUMBER: _ClassVar[int]
    EXPIRES_IN_FIELD_NUMBER: _ClassVar[int]
    SCOPES_FIELD_NUMBER: _ClassVar[int]
    access_token: str
    issued_token_type: str
    token_type: str
    expires_in: int
    scopes: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, access_token: _Optional[str] = ..., issued_token_type: _Optional[str] = ..., token_type: _Optional[str] = ..., expires_in: _Optional[int] = ..., scopes: _Optional[_Iterable[str]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/audit/v2/response.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.rpc import code_pb2 as google_dot_rpc_dot_code__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1enebius/audit/v2/response.proto\x12\x0fnebius.audit.v2\x1a\x1cgoogle/protobuf/struct.proto\x1a\x15google/rpc/code.proto\"\x95\x01\n\x08Response\x12\x31\n\x0bstatus_code\x18\x01 \x01(\x0e\x32\x10.google.rpc.CodeR\nstatusCode\x12#\n\rerror_message\x18\x02 \x01(\tR\x0c\x65rrorMessage\x12\x31\n\x07payload\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructR\x07payloadBX\n\x16\x61i.nebius.pub.audit.v2B\rResponseProtoP\x01Z-github.com/nebius/gosdk/proto/nebius/audit/v2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.audit.v2.response_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ai.nebius.pub.audit.v2B\rResponseProtoP\001Z-github.com/nebius/gosdk/proto/nebius/audit/v2'
  _globals['_RESPONSE']._serialized_start=105
  _globals['_RESPONSE']._serialized_end=254
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius.mk8s.v1alpha1 import instance_template_pb2 as _instance_template_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ConditionStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    CONDITION_STATUS_UNSPECIFIED: _ClassVar[ConditionStatus]
    TRUE: _ClassVar[ConditionStatus]
    FALSE: _ClassVar[ConditionStatus]
    UNKNOWN: _ClassVar[ConditionStatus]
CONDITION_STATUS_UNSPECIFIED: ConditionStatus
TRUE: ConditionStatus
FALSE: ConditionStatus
UNKNOWN: ConditionStatus

class NodeGroup(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: NodeGroupSpec
    status: NodeGroupStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[NodeGroupSpec, _Mapping]] = ..., status: _Optional[_Union[NodeGroupStatus, _Mapping]] = ...) -> None: ...

class NodeGroupSpec(_message.Message):
    __slots__ = ["version", "fixed_node_count", "autoscaling", "template", "strategy", "auto_repair"]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    FIXED_NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    AUTOSCALING_FIELD_NUMBER: _ClassVar[int]
    TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    STRATEGY_FIELD_NUMBER: _ClassVar[int]
    AUTO_REPAIR_FIELD_NUMBER: _ClassVar[int]
    version: str
    fixed_node_count: int
    autoscaling: NodeGroupAutoscalingSpec
    template: NodeTemplate
    strategy: NodeGroupDeploymentStrategy
    auto_repair: NodeGroupAutoRepairSpec
    def __init__(self, version: _Optional[str] = ..., fixed_node_count: _Optional[int] = ..., autoscaling: _Optional[_Union[NodeGroupAutoscalingSpec, _Mapping]] = ..., template: _Optional[_Union[NodeTemplate, _Mapping]] = ..., strategy: _Optional[_Union[NodeGroupDeploymentStrategy, _Mapping]] = ..., auto_repair: _Optional[_Union[NodeGroupAutoRepairSpec, _Mapping]] = ...) -> None: ...

class NodeTemplate(_message.Message):
    __slots__ = ["metadata", "taints", "resources", "boot_disk", "gpu_settings", "os", "gpu_cluster", "network_interfaces", "filesystems", "cloud_init_user_data", "service_account_id", "preemptible"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    TAINTS_FIELD_NUMBER: _ClassVar[int]
    RESOURCES_FIELD_NUMBER: _ClassVar[int]
    BOOT_DISK_FIELD_NUMBER: _ClassVar[int]
    GPU_SETTINGS_FIELD_NUMBER: _ClassVar[int]
    OS_FIELD_NUMBER: _ClassVar[int]
    GPU_CLUSTER_FIELD_NUMBER: _ClassVar[int]
    NETWORK_INTERFACES_FIELD_NUMBER: _ClassVar[int]
    FILESYSTEMS_FIELD_NUMBER: _ClassVar[int]
    CLOUD_INIT_USER_DATA_FIELD_NUMBER: _ClassVar[int]
    SERVICE_ACCOUNT_ID_FIELD_NUMBER: _ClassVar[int]
    PREEMPTIBLE_FIELD_NUMBER: _ClassVar[int]
    metadata: NodeMetadataTemplate
    taints: _containers.RepeatedCompositeFieldContainer[NodeTaint]
    resources: _instance_template_pb2.ResourcesSpec
    boot_disk: _instance_template_pb2.DiskSpec
    gpu_settings: GpuSettings
    os: str
    gpu_cluster: GpuClusterSpec
    network_interfaces: _containers.RepeatedCompositeFieldContainer[NetworkInterfaceTemplate]
    filesystems: _containers.RepeatedCompositeFieldContainer[AttachedFilesystemSpec]
    cloud_init_user_data: str
    service_account_id: str
    preemptible: PreemptibleSpec
    def __init__(self, metadata: _Optional[_Union[NodeMetadataTemplate, _Mapping]] = ..., taints: _Optional[_Iterable[_Union[NodeTaint, _Mapping]]] = ..., resources: _Optional[_Union[_instance_template_pb2.ResourcesSpec, _Mapping]] = ..., boot_disk: _Optional[_Union[_instance_template_pb2.DiskSpec, _Mapping]] = ..., gpu_settings: _Optional[_Union[GpuSettings, _Mapping]] = ..., os: _Optional[str] = ..., gpu_cluster: _Optional[_Union[GpuClusterSpec, _Mapping]] = ..., network_interfaces: _Optional[_Iterable[_Union[NetworkInterfaceTemplate, _Mapping]]] = ..., filesystems: _Optional[_Iterable[_Union[AttachedFilesystemSpec, _Mapping]]] = ..., cloud_init_user_data: _Optional[str] = ..., service_account_id: _Optional[str] = ..., preemptible: _Optional[_Union[PreemptibleSpec, _Mapping]] = ...) -> None: ...

class NodeMetadataTemplate(_message.Message):
    __slots__ = ["labels"]
    class LabelsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    LABELS_FIELD_NUMBER: _ClassVar[int]
    labels: _containers.ScalarMap[str, str]
    def __init__(self, labels: _Optional[_Mapping[str, str]] = ...) -> None: ...

class GpuSettings(_message.Message):
    __slots__ = ["drivers_preset"]
    DRIVERS_PRESET_FIELD_NUMBER: _ClassVar[int]
    drivers_preset: str
    def __init__(self, drivers_preset: _Optional[str] = ...) -> None: ...

class GpuClusterSpec(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class NetworkInterfaceTemplate(_message.Message):
    __slots__ = ["public_ip_address", "subnet_id"]
    PUBLIC_IP_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    SUBNET_ID_FIELD_NUMBER: _ClassVar[int]
    public_ip_address: PublicIPAddress
    subnet_id: str
    def __init__(self, public_ip_address: _Optional[_Union[PublicIPAddress, _Mapping]] = ..., subnet_id: _Optional[str] = ...) -> None: ...

class PublicIPAddress(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class AttachedFilesystemSpec(_message.Message):
    __slots__ = ["attach_mode", "device_name", "existing_filesystem"]
    class AttachMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[AttachedFilesystemSpec.AttachMode]
        READ_ONLY: _ClassVar[AttachedFilesystemSpec.AttachMode]
        READ_WRITE: _ClassVar[AttachedFilesystemSpec.AttachMode]
    UNSPECIFIED: AttachedFilesystemSpec.AttachMode
    READ_ONLY: AttachedFilesystemSpec.AttachMode
    READ_WRITE: AttachedFilesystemSpec.AttachMode
    ATTACH_MODE_FIELD_NUMBER: _ClassVar[int]
    DEVICE_NAME_FIELD_NUMBER: _ClassVar[int]
    EXISTING_FILESYSTEM_FIELD_NUMBER: _ClassVar[int]
    attach_mode: AttachedFilesystemSpec.AttachMode
    device_name: str
    existing_filesystem: ExistingFilesystem
    def __init__(self, attach_mode: _Optional[_Union[AttachedFilesystemSpec.AttachMode, str]] = ..., device_name: _Optional[str] = ..., existing_filesystem: _Optional[_Union[ExistingFilesystem, _Mapping]] = ...) -> None: ...

class ExistingFilesystem(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class NodeGroupAutoscalingSpec(_message.Message):
    __slots__ = ["min_node_count", "max_node_count"]
    MIN_NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    MAX_NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    min_node_count: int
    max_node_count: int
    def __init__(self, min_node_count: _Optional[int] = ..., max_node_count: _Optional[int] = ...) -> None: ...

class PreemptibleSpec(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class NodeTaint(_message.Message):
    __slots__ = ["key", "value", "effect"]
    class Effect(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        EFFECT_UNSPECIFIED: _ClassVar[NodeTaint.Effect]
        NO_EXECUTE: _ClassVar[NodeTaint.Effect]
        NO_SCHEDULE: _ClassVar[NodeTaint.Effect]
        PREFER_NO_SCHEDULE: _ClassVar[NodeTaint.Effect]
    EFFECT_UNSPECIFIED: NodeTaint.Effect
    NO_EXECUTE: NodeTaint.Effect
    NO_SCHEDULE: NodeTaint.Effect
    PREFER_NO_SCHEDULE: NodeTaint.Effect
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    EFFECT_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: str
    effect: NodeTaint.Effect
    def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ..., effect: _Optional[_Union[NodeTaint.Effect, str]] = ...) -> None: ...

class NodeGroupDeploymentStrategy(_message.Message):
    __slots__ = ["max_unavailable", "max_surge", "drain_timeout"]
    MAX_UNAVAILABLE_FIELD_NUMBER: _ClassVar[int]
    MAX_SURGE_FIELD_NUMBER: _ClassVar[int]
    DRAIN_TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    max_unavailable: PercentOrCount
    max_surge: PercentOrCount
    drain_timeout: _duration_pb2.Duration
    def __init__(self, max_unavailable: _Optional[_Union[PercentOrCount, _Mapping]] = ..., max_surge: _Optional[_Union[PercentOrCount, _Mapping]] = ..., drain_timeout: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class PercentOrCount(_message.Message):
    __slots__ = ["percent", "count"]
    PERCENT_FIELD_NUMBER: _ClassVar[int]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    percent: int
    count: int
    def __init__(self, percent: _Optional[int] = ..., count: _Optional[int] = ...) -> None: ...

class NodeGroupAutoRepairSpec(_message.Message):
    __slots__ = ["conditions"]
    CONDITIONS_FIELD_NUMBER: _ClassVar[int]
    conditions: _containers.RepeatedCompositeFieldContainer[NodeAutoRepairCondition]
    def __init__(self, conditions: _Optional[_Iterable[_Union[NodeAutoRepairCondition, _Mapping]]] = ...) -> None: ...

class NodeAutoRepairCondition(_message.Message):
    __slots__ = ["type", "status", "timeout", "disabled"]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    DISABLED_FIELD_NUMBER: _ClassVar[int]
    type: str
    status: ConditionStatus
    timeout: _duration_pb2.Duration
    disabled: bool
    def __init__(self, type: _Optional[str] = ..., status: _Optional[_Union[ConditionStatus, str]] = ..., timeout: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., disabled: bool = ...) -> None: ...

class NodeGroupStatus(_message.Message):
    __slots__ = ["state", "version", "target_node_count", "node_count", "outdated_node_count", "ready_node_count", "reconciling"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[NodeGroupStatus.State]
        PROVISIONING: _ClassVar[NodeGroupStatus.State]
        RUNNING: _ClassVar[NodeGroupStatus.State]
        DELETING: _ClassVar[NodeGroupStatus.State]
    STATE_UNSPECIFIED: NodeGroupStatus.State
    PROVISIONING: NodeGroupStatus.State
    RUNNING: NodeGroupStatus.State
    DELETING: NodeGroupStatus.State
    STATE_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    TARGET_NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    OUTDATED_NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    READY_NODE_COUNT_FIELD_NUMBER: _ClassVar[int]
    RECONCILING_FIELD_NUMBER: _ClassVar[int]
    state: NodeGroupStatus.State
    version: str
    target_node_count: int
    node_count: int
    outdated_node_count: int
    ready_node_count: int
    reconciling: bool
    def __init__(self, state: _Optional[_Union[NodeGroupStatus.State, str]] = ..., version: _Optional[str] = ..., target_node_count: _Optional[int] = ..., node_count: _Optional[int] = ..., outdated_node_count: _Optional[int] = ..., ready_node_count: _Optional[int] = ..., reconciling: bool = ...) -> None: ...



"""Authorization interfaces used by the async SDK.

This module defines two abstract interfaces used by the SDK to perform
request-time authorization.

Typical usage within the request lifecycle
------------------------------------------

The :class:`Provider` is the object passed to request constructors and prior to sending
an RPC the request layer calls :meth:`Provider.authenticator()` to obtain an
:class:`Authenticator`.

The :class:`Authenticator` instance returned by the provider is treated as a
per-request authorizer: it is responsible for ensuring the
provided :class:`nebius.base.metadata.Metadata` contains the headers needed
for that request only (for example an Authorization bearer token). If
an authentication-related failure occurs, the request layer will consult
``Authenticator.can_retry`` and will retry with the same authenticator.
"""

from abc import ABC, abstractmethod

from nebius.base.metadata import Metadata


class Authenticator(ABC):
    """Abstract interface for performing per-request authentication.

    Subclasses must implement :meth:`authenticate` and may implement
    :meth:`can_retry` to indicate whether authentication failures should be
    retried, eg code UNAUTHENTICATED and number of calls to :meth:`authenticate` < 3.
    """

    @abstractmethod
    async def authenticate(
        self,
        metadata: Metadata,
        timeout: float | None = None,
        options: dict[str, str] | None = None,
    ) -> None:
        """Authenticate by modifying the ``metadata`` before sending an RPC.

        :param metadata: The metadata mapping that will be sent with the RPC.
            Implementations may mutate this mapping in-place to add or update
            authentication headers (for example the Authorization header).
        :type metadata: :class:`nebius.base.metadata.Metadata`
        :param timeout: Optional authentication timeout in seconds. Implementations
            must not exceed this timeout during the whole authentication process.
        :type timeout: optional `float`
        :param options: Optional, implementation-specific options passed from
            the request layer.
        :type options: optional ``dict[str, str]``
        """
        raise NotImplementedError("Method not implemented!")

    @abstractmethod
    def can_retry(
        self,
        err: Exception,
        options: dict[str, str] | None = None,
    ) -> bool:
        """Indicate whether a failed authentication attempt or failed request may be
        retried with a fresh authentication, calling to :meth:`authenticate`.

        :param err: The exception raised during authentication or while the
            RPC was in-flight. Implementations inspect the exception to
            determine if a retry (for example after refreshing a token) is
            likely to succeed.
        :type err: :class:`Exception`
        :param options: Optional implementation-specific options.
        :type options: optional ``dict[str, str]``
        :returns: ``True`` when the authentication should be retried, otherwise
            ``False``.
        :rtype: bool
        """
        return False


class Provider(ABC):
    """Factory abstraction that supplies an :class:`Authenticator`.

    Typical usage within the request lifecycle
    ------------------------------------------

    The :class:`Provider` is the object passed to request constructors and prior to
    sending an RPC the request layer calls :meth:`Provider.authenticator()` to obtain an
    :class:`Authenticator`.

    The :class:`Authenticator` instance returned by the provider is treated as a
    per-request authorizer: it is responsible for ensuring the
    provided :class:`nebius.base.metadata.Metadata` contains the headers needed
    for that request only (for example an Authorization bearer token). If
    an authentication-related failure occurs, the request layer will consult
    ``Authenticator.can_retry`` and will retry with the same authenticator.

    Example
    -------

    A minimal example showing how a provider might be passed into an SDK or
    request layer. The actual SDK in this repo accepts a provider via the
    ``credentials`` parameter; the example below demonstrates the intent and
    typical usage::

        from nebius.sdk import SDK
        from nebius.aio.authorization import Authenticator, Provider

        class MyAuthenticator(Authenticator):
            async def authenticate(self, metadata, timeout=None, options=None):
                metadata.add("Authorization", "Bearer my-static-token")

        class MyProvider(Provider):
            def authenticator(self):
                return MyAuthenticator()

        provider = MyProvider()
        sdk = SDK(credentials=provider)  # SDK initialisation (illustrative)

    """

    @abstractmethod
    def authenticator(self) -> Authenticator:
        """Return a fresh per-request :class:`Authenticator` instance.

        :returns: An authenticator instance.
        """
        raise NotImplementedError("Method not implemented!")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/common/error/v1alpha1/common_errors.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0nebius/common/error/v1alpha1/common_errors.proto\x12\x1cnebius.common.error.v1alpha1\x1a\x18nebius/annotations.proto\"\x9d\x01\n\nBadRequest\x12R\n\nviolations\x18\x01 \x03(\x0b\x32\x32.nebius.common.error.v1alpha1.BadRequest.ViolationR\nviolations\x1a;\n\tViolation\x12\x14\n\x05\x66ield\x18\x01 \x01(\tR\x05\x66ield\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\"M\n\x10\x42\x61\x64ResourceState\x12\x1f\n\x0bresource_id\x18\x01 \x01(\tR\nresourceId\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\"3\n\x10ResourceNotFound\x12\x1f\n\x0bresource_id\x18\x01 \x01(\tR\nresourceId\"8\n\x15ResourceAlreadyExists\x12\x1f\n\x0bresource_id\x18\x01 \x01(\tR\nresourceId\"M\n\x10ResourceConflict\x12\x1f\n\x0bresource_id\x18\x01 \x01(\tR\nresourceId\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\"\x8d\x01\n\x10OperationAborted\x12!\n\x0coperation_id\x18\x01 \x01(\tR\x0boperationId\x12\x35\n\x17\x61\x62orted_by_operation_id\x18\x02 \x01(\tR\x14\x61\x62ortedByOperationId\x12\x1f\n\x0bresource_id\x18\x03 \x01(\tR\nresourceId\"@\n\nOutOfRange\x12\x1c\n\trequested\x18\x01 \x01(\tR\trequested\x12\x14\n\x05limit\x18\x02 \x01(\tR\x05limit\"3\n\x10PermissionDenied\x12\x1f\n\x0bresource_id\x18\x01 \x01(\tR\nresourceId\"I\n\rInternalError\x12\x1d\n\nrequest_id\x18\x01 \x01(\tR\trequestId\x12\x19\n\x08trace_id\x18\x02 \x01(\tR\x07traceId\"/\n\x0fTooManyRequests\x12\x1c\n\tviolation\x18\x01 \x01(\tR\tviolation\"\xd5\x01\n\x0cQuotaFailure\x12T\n\nviolations\x18\x01 \x03(\x0b\x32\x34.nebius.common.error.v1alpha1.QuotaFailure.ViolationR\nviolations\x1ao\n\tViolation\x12\x14\n\x05quota\x18\x01 \x01(\tR\x05quota\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x14\n\x05limit\x18\x03 \x01(\tR\x05limit\x12\x1c\n\trequested\x18\x04 \x01(\tR\trequestedB\x92\x01\n#ai.nebius.pub.common.error.v1alpha1B\x11\x43ommonErrorsProtoP\x01Z:github.com/nebius/gosdk/proto/nebius/common/error/v1alpha1\xb8\x01\x01\xd2J\x16\x12\x14migrate to common/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.common.error.v1alpha1.common_errors_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#ai.nebius.pub.common.error.v1alpha1B\021CommonErrorsProtoP\001Z:github.com/nebius/gosdk/proto/nebius/common/error/v1alpha1\270\001\001\322J\026\022\024migrate to common/v1'
  _globals['_BADREQUEST']._serialized_start=109
  _globals['_BADREQUEST']._serialized_end=266
  _globals['_BADREQUEST_VIOLATION']._serialized_start=207
  _globals['_BADREQUEST_VIOLATION']._serialized_end=266
  _globals['_BADRESOURCESTATE']._serialized_start=268
  _globals['_BADRESOURCESTATE']._serialized_end=345
  _globals['_RESOURCENOTFOUND']._serialized_start=347
  _globals['_RESOURCENOTFOUND']._serialized_end=398
  _globals['_RESOURCEALREADYEXISTS']._serialized_start=400
  _globals['_RESOURCEALREADYEXISTS']._serialized_end=456
  _globals['_RESOURCECONFLICT']._serialized_start=458
  _globals['_RESOURCECONFLICT']._serialized_end=535
  _globals['_OPERATIONABORTED']._serialized_start=538
  _globals['_OPERATIONABORTED']._serialized_end=679
  _globals['_OUTOFRANGE']._serialized_start=681
  _globals['_OUTOFRANGE']._serialized_end=745
  _globals['_PERMISSIONDENIED']._serialized_start=747
  _globals['_PERMISSIONDENIED']._serialized_end=798
  _globals['_INTERNALERROR']._serialized_start=800
  _globals['_INTERNALERROR']._serialized_end=873
  _globals['_TOOMANYREQUESTS']._serialized_start=875
  _globals['_TOOMANYREQUESTS']._serialized_end=922
  _globals['_QUOTAFAILURE']._serialized_start=925
  _globals['_QUOTAFAILURE']._serialized_end=1138
  _globals['_QUOTAFAILURE_VIOLATION']._serialized_start=1027
  _globals['_QUOTAFAILURE_VIOLATION']._serialized_end=1138
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/federation_certificate.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*nebius/iam/v1/federation_certificate.proto\x12\rnebius.iam.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a\x18nebius/annotations.proto\"\xf5\x01\n\x15\x46\x65\x64\x65rationCertificate\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x44\n\x04spec\x18\x02 \x01(\x0b\x32(.nebius.iam.v1.FederationCertificateSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12H\n\x06status\x18\x03 \x01(\x0b\x32*.nebius.iam.v1.FederationCertificateStatusB\x04\xbaJ\x01\x05R\x06status:\x04\xbaJ\x01\x02\"W\n\x19\x46\x65\x64\x65rationCertificateSpec\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\x12\x18\n\x04\x64\x61ta\x18\x02 \x01(\tB\x04\xbaJ\x01\x02R\x04\x64\x61ta\"\xcb\x02\n\x1b\x46\x65\x64\x65rationCertificateStatus\x12\x46\n\x05state\x18\x01 \x01(\x0e\x32\x30.nebius.iam.v1.FederationCertificateStatus.StateR\x05state\x12\x1c\n\talgorithm\x18\x03 \x01(\tR\talgorithm\x12\x19\n\x08key_size\x18\x04 \x01(\x03R\x07keySize\x12\x39\n\nnot_before\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tnotBefore\x12\x37\n\tnot_after\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x08notAfter\"7\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x01\x12\x0b\n\x07\x45XPIRED\x10\x02\x42\x61\n\x14\x61i.nebius.pub.iam.v1B\x1a\x46\x65\x64\x65rationCertificateProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.federation_certificate_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\032FederationCertificateProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _FEDERATIONCERTIFICATE.fields_by_name['metadata']._options = None
  _FEDERATIONCERTIFICATE.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _FEDERATIONCERTIFICATE.fields_by_name['spec']._options = None
  _FEDERATIONCERTIFICATE.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _FEDERATIONCERTIFICATE.fields_by_name['status']._options = None
  _FEDERATIONCERTIFICATE.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _FEDERATIONCERTIFICATE._options = None
  _FEDERATIONCERTIFICATE._serialized_options = b'\272J\001\002'
  _FEDERATIONCERTIFICATESPEC.fields_by_name['data']._options = None
  _FEDERATIONCERTIFICATESPEC.fields_by_name['data']._serialized_options = b'\272J\001\002'
  _globals['_FEDERATIONCERTIFICATE']._serialized_start=183
  _globals['_FEDERATIONCERTIFICATE']._serialized_end=428
  _globals['_FEDERATIONCERTIFICATESPEC']._serialized_start=430
  _globals['_FEDERATIONCERTIFICATESPEC']._serialized_end=517
  _globals['_FEDERATIONCERTIFICATESTATUS']._serialized_start=520
  _globals['_FEDERATIONCERTIFICATESTATUS']._serialized_end=851
  _globals['_FEDERATIONCERTIFICATESTATUS_STATE']._serialized_start=796
  _globals['_FEDERATIONCERTIFICATESTATUS_STATE']._serialized_end=851
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/mk8s/v1/node_group.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import resource_event_pb2 as nebius_dot_common_dot_v1_dot_resource__event__pb2
from nebius.api.nebius.mk8s.v1 import instance_template_pb2 as nebius_dot_mk8s_dot_v1_dot_instance__template__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fnebius/mk8s/v1/node_group.proto\x12\x0enebius.mk8s.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a%nebius/common/v1/resource_event.proto\x1a&nebius/mk8s/v1/instance_template.proto\"\xb7\x01\n\tNodeGroup\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x31\n\x04spec\x18\x02 \x01(\x0b\x32\x1d.nebius.mk8s.v1.NodeGroupSpecR\x04spec\x12\x37\n\x06status\x18\x03 \x01(\x0b\x32\x1f.nebius.mk8s.v1.NodeGroupStatusR\x06status\"\xa6\x03\n\rNodeGroupSpec\x12,\n\x07version\x18\x01 \x01(\tB\x12\xbaH\x0fr\r2\x0b|^\\d\\.\\d\\d$R\x07version\x12\x35\n\x10\x66ixed_node_count\x18\x02 \x01(\x03\x42\t\xbaH\x06\"\x04\x18\x64(\x00H\x00R\x0e\x66ixedNodeCount\x12L\n\x0b\x61utoscaling\x18\x05 \x01(\x0b\x32(.nebius.mk8s.v1.NodeGroupAutoscalingSpecH\x00R\x0b\x61utoscaling\x12@\n\x08template\x18\x03 \x01(\x0b\x32\x1c.nebius.mk8s.v1.NodeTemplateB\x06\xbaH\x03\xc8\x01\x01R\x08template\x12G\n\x08strategy\x18\x04 \x01(\x0b\x32+.nebius.mk8s.v1.NodeGroupDeploymentStrategyR\x08strategy\x12H\n\x0b\x61uto_repair\x18\x06 \x01(\x0b\x32\'.nebius.mk8s.v1.NodeGroupAutoRepairSpecR\nautoRepairB\r\n\x04size\x12\x05\xbaH\x02\x08\x01\"\xf6\x05\n\x0cNodeTemplate\x12@\n\x08metadata\x18\x01 \x01(\x0b\x32$.nebius.mk8s.v1.NodeMetadataTemplateR\x08metadata\x12;\n\x06taints\x18\x02 \x03(\x0b\x32\x19.nebius.mk8s.v1.NodeTaintB\x08\xbaH\x05\x92\x01\x02\x10\x64R\x06taints\x12\x43\n\tresources\x18\x03 \x01(\x0b\x32\x1d.nebius.mk8s.v1.ResourcesSpecB\x06\xbaH\x03\xc8\x01\x01R\tresources\x12;\n\tboot_disk\x18\t \x01(\x0b\x32\x18.nebius.mk8s.v1.DiskSpecB\x04\xbaJ\x01\x07R\x08\x62ootDisk\x12>\n\x0cgpu_settings\x18\r \x01(\x0b\x32\x1b.nebius.mk8s.v1.GpuSettingsR\x0bgpuSettings\x12\x0e\n\x02os\x18\x10 \x01(\tR\x02os\x12?\n\x0bgpu_cluster\x18\x04 \x01(\x0b\x32\x1e.nebius.mk8s.v1.GpuClusterSpecR\ngpuCluster\x12]\n\x12network_interfaces\x18\x05 \x03(\x0b\x32(.nebius.mk8s.v1.NetworkInterfaceTemplateB\x04\xbaJ\x01\x07R\x11networkInterfaces\x12H\n\x0b\x66ilesystems\x18\x07 \x03(\x0b\x32&.nebius.mk8s.v1.AttachedFilesystemSpecR\x0b\x66ilesystems\x12\x34\n\x14\x63loud_init_user_data\x18\x06 \x01(\tB\x03\xc0J\x01R\x11\x63loudInitUserData\x12,\n\x12service_account_id\x18\n \x01(\tR\x10serviceAccountId\x12G\n\x0bpreemptible\x18\x0f \x01(\x0b\x32\x1f.nebius.mk8s.v1.PreemptibleSpecB\x04\xbaJ\x01\x06R\x0bpreemptible\"\xa5\x01\n\x14NodeMetadataTemplate\x12R\n\x06labels\x18\x01 \x03(\x0b\x32\x30.nebius.mk8s.v1.NodeMetadataTemplate.LabelsEntryB\x08\xbaH\x05\x9a\x01\x02\x10\x64R\x06labels\x1a\x39\n\x0bLabelsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"<\n\x0bGpuSettings\x12-\n\x0e\x64rivers_preset\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\rdriversPreset\" \n\x0eGpuClusterSpec\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\x90\x01\n\x18NetworkInterfaceTemplate\x12Q\n\x11public_ip_address\x18\x01 \x01(\x0b\x32\x1f.nebius.mk8s.v1.PublicIPAddressB\x04\xbaJ\x01\x06R\x0fpublicIpAddress\x12!\n\tsubnet_id\x18\x03 \x01(\tB\x04\xbaJ\x01\x07R\x08subnetId\"\x11\n\x0fPublicIPAddress\"\xbd\x02\n\x16\x41ttachedFilesystemSpec\x12Z\n\x0b\x61ttach_mode\x18\x01 \x01(\x0e\x32\x31.nebius.mk8s.v1.AttachedFilesystemSpec.AttachModeB\x06\xbaH\x03\xc8\x01\x01R\nattachMode\x12#\n\tmount_tag\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08mountTag\x12U\n\x13\x65xisting_filesystem\x18\x03 \x01(\x0b\x32\".nebius.mk8s.v1.ExistingFilesystemH\x00R\x12\x65xistingFilesystem\"<\n\nAttachMode\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\r\n\tREAD_ONLY\x10\x01\x12\x0e\n\nREAD_WRITE\x10\x02\x42\r\n\x04type\x12\x05\xbaH\x02\x08\x01\",\n\x12\x45xistingFilesystem\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"\x8f\x02\n\x18NodeGroupAutoscalingSpec\x12/\n\x0emin_node_count\x18\x01 \x01(\x03\x42\t\xbaH\x06\"\x04\x18\x64(\x00R\x0cminNodeCount\x12/\n\x0emax_node_count\x18\x02 \x01(\x03\x42\t\xbaH\x06\"\x04\x18\x64(\x00R\x0cmaxNodeCount:\x90\x01\xbaH\x8c\x01\x1a\x89\x01\n!autoscaling.protovalidate.message\x12\x38min_node_count must be less or equal than max_node_count\x1a*this.min_node_count <= this.max_node_count\"\xe0\x01\n\tNodeTaint\x12\x18\n\x03key\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x03key\x12\x1c\n\x05value\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x05value\x12@\n\x06\x65\x66\x66\x65\x63t\x18\x03 \x01(\x0e\x32 .nebius.mk8s.v1.NodeTaint.EffectB\x06\xbaH\x03\xc8\x01\x01R\x06\x65\x66\x66\x65\x63t\"Y\n\x06\x45\x66\x66\x65\x63t\x12\x16\n\x12\x45\x46\x46\x45\x43T_UNSPECIFIED\x10\x00\x12\x0e\n\nNO_EXECUTE\x10\x01\x12\x0f\n\x0bNO_SCHEDULE\x10\x02\x12\x16\n\x12PREFER_NO_SCHEDULE\x10\x03\"\xed\x01\n\x1bNodeGroupDeploymentStrategy\x12G\n\x0fmax_unavailable\x18\x01 \x01(\x0b\x32\x1e.nebius.mk8s.v1.PercentOrCountR\x0emaxUnavailable\x12;\n\tmax_surge\x18\x02 \x01(\x0b\x32\x1e.nebius.mk8s.v1.PercentOrCountR\x08maxSurge\x12H\n\rdrain_timeout\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xbaH\x05\xaa\x01\x02\x32\x00R\x0c\x64rainTimeout\"h\n\x0ePercentOrCount\x12%\n\x07percent\x18\x01 \x01(\x03\x42\t\xbaH\x06\"\x04\x18\x64(\x00H\x00R\x07percent\x12\x1f\n\x05\x63ount\x18\x02 \x01(\x03\x42\x07\xbaH\x04\"\x02(\x00H\x00R\x05\x63ountB\x0e\n\x05value\x12\x05\xbaH\x02\x08\x01\"b\n\x17NodeGroupAutoRepairSpec\x12G\n\nconditions\x18\x04 \x03(\x0b\x32\'.nebius.mk8s.v1.NodeAutoRepairConditionR\nconditions\"\xd5\x01\n\x17NodeAutoRepairCondition\x12\x1a\n\x04type\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04type\x12\x37\n\x06status\x18\x02 \x01(\x0e\x32\x1f.nebius.mk8s.v1.ConditionStatusR\x06status\x12\x35\n\x07timeout\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00R\x07timeout\x12\x1c\n\x08\x64isabled\x18\x04 \x01(\x08H\x00R\x08\x64isabledB\x10\n\x07trigger\x12\x05\xbaH\x02\x08\x01\"\x11\n\x0fPreemptibleSpec\"\xbe\x03\n\x0fNodeGroupStatus\x12;\n\x05state\x18\x01 \x01(\x0e\x32%.nebius.mk8s.v1.NodeGroupStatus.StateR\x05state\x12\x18\n\x07version\x18\x02 \x01(\tR\x07version\x12*\n\x11target_node_count\x18\x03 \x01(\x03R\x0ftargetNodeCount\x12\x1d\n\nnode_count\x18\x04 \x01(\x03R\tnodeCount\x12.\n\x13outdated_node_count\x18\x05 \x01(\x03R\x11outdatedNodeCount\x12(\n\x10ready_node_count\x18\x06 \x01(\x03R\x0ereadyNodeCount\x12@\n\x06\x65vents\x18= \x03(\x0b\x32(.nebius.common.v1.RecurrentResourceEventR\x06\x65vents\x12 \n\x0breconciling\x18\x64 \x01(\x08R\x0breconciling\"K\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x10\n\x0cPROVISIONING\x10\x01\x12\x0b\n\x07RUNNING\x10\x02\x12\x0c\n\x08\x44\x45LETING\x10\x03*U\n\x0f\x43onditionStatus\x12 \n\x1c\x43ONDITION_STATUS_UNSPECIFIED\x10\x00\x12\x08\n\x04TRUE\x10\x01\x12\t\n\x05\x46\x41LSE\x10\x02\x12\x0b\n\x07UNKNOWN\x10\x03\x42W\n\x15\x61i.nebius.pub.mk8s.v1B\x0eNodeGroupProtoP\x01Z,github.com/nebius/gosdk/proto/nebius/mk8s/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.mk8s.v1.node_group_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025ai.nebius.pub.mk8s.v1B\016NodeGroupProtoP\001Z,github.com/nebius/gosdk/proto/nebius/mk8s/v1'
  _NODEGROUPSPEC.oneofs_by_name['size']._options = None
  _NODEGROUPSPEC.oneofs_by_name['size']._serialized_options = b'\272H\002\010\001'
  _NODEGROUPSPEC.fields_by_name['version']._options = None
  _NODEGROUPSPEC.fields_by_name['version']._serialized_options = b'\272H\017r\r2\013|^\\d\\.\\d\\d$'
  _NODEGROUPSPEC.fields_by_name['fixed_node_count']._options = None
  _NODEGROUPSPEC.fields_by_name['fixed_node_count']._serialized_options = b'\272H\006\"\004\030d(\000'
  _NODEGROUPSPEC.fields_by_name['template']._options = None
  _NODEGROUPSPEC.fields_by_name['template']._serialized_options = b'\272H\003\310\001\001'
  _NODETEMPLATE.fields_by_name['taints']._options = None
  _NODETEMPLATE.fields_by_name['taints']._serialized_options = b'\272H\005\222\001\002\020d'
  _NODETEMPLATE.fields_by_name['resources']._options = None
  _NODETEMPLATE.fields_by_name['resources']._serialized_options = b'\272H\003\310\001\001'
  _NODETEMPLATE.fields_by_name['boot_disk']._options = None
  _NODETEMPLATE.fields_by_name['boot_disk']._serialized_options = b'\272J\001\007'
  _NODETEMPLATE.fields_by_name['network_interfaces']._options = None
  _NODETEMPLATE.fields_by_name['network_interfaces']._serialized_options = b'\272J\001\007'
  _NODETEMPLATE.fields_by_name['cloud_init_user_data']._options = None
  _NODETEMPLATE.fields_by_name['cloud_init_user_data']._serialized_options = b'\300J\001'
  _NODETEMPLATE.fields_by_name['preemptible']._options = None
  _NODETEMPLATE.fields_by_name['preemptible']._serialized_options = b'\272J\001\006'
  _NODEMETADATATEMPLATE_LABELSENTRY._options = None
  _NODEMETADATATEMPLATE_LABELSENTRY._serialized_options = b'8\001'
  _NODEMETADATATEMPLATE.fields_by_name['labels']._options = None
  _NODEMETADATATEMPLATE.fields_by_name['labels']._serialized_options = b'\272H\005\232\001\002\020d'
  _GPUSETTINGS.fields_by_name['drivers_preset']._options = None
  _GPUSETTINGS.fields_by_name['drivers_preset']._serialized_options = b'\272H\003\310\001\001'
  _NETWORKINTERFACETEMPLATE.fields_by_name['public_ip_address']._options = None
  _NETWORKINTERFACETEMPLATE.fields_by_name['public_ip_address']._serialized_options = b'\272J\001\006'
  _NETWORKINTERFACETEMPLATE.fields_by_name['subnet_id']._options = None
  _NETWORKINTERFACETEMPLATE.fields_by_name['subnet_id']._serialized_options = b'\272J\001\007'
  _ATTACHEDFILESYSTEMSPEC.oneofs_by_name['type']._options = None
  _ATTACHEDFILESYSTEMSPEC.oneofs_by_name['type']._serialized_options = b'\272H\002\010\001'
  _ATTACHEDFILESYSTEMSPEC.fields_by_name['attach_mode']._options = None
  _ATTACHEDFILESYSTEMSPEC.fields_by_name['attach_mode']._serialized_options = b'\272H\003\310\001\001'
  _ATTACHEDFILESYSTEMSPEC.fields_by_name['mount_tag']._options = None
  _ATTACHEDFILESYSTEMSPEC.fields_by_name['mount_tag']._serialized_options = b'\272H\003\310\001\001'
  _EXISTINGFILESYSTEM.fields_by_name['id']._options = None
  _EXISTINGFILESYSTEM.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _NODEGROUPAUTOSCALINGSPEC.fields_by_name['min_node_count']._options = None
  _NODEGROUPAUTOSCALINGSPEC.fields_by_name['min_node_count']._serialized_options = b'\272H\006\"\004\030d(\000'
  _NODEGROUPAUTOSCALINGSPEC.fields_by_name['max_node_count']._options = None
  _NODEGROUPAUTOSCALINGSPEC.fields_by_name['max_node_count']._serialized_options = b'\272H\006\"\004\030d(\000'
  _NODEGROUPAUTOSCALINGSPEC._options = None
  _NODEGROUPAUTOSCALINGSPEC._serialized_options = b'\272H\214\001\032\211\001\n!autoscaling.protovalidate.message\0228min_node_count must be less or equal than max_node_count\032*this.min_node_count <= this.max_node_count'
  _NODETAINT.fields_by_name['key']._options = None
  _NODETAINT.fields_by_name['key']._serialized_options = b'\272H\003\310\001\001'
  _NODETAINT.fields_by_name['value']._options = None
  _NODETAINT.fields_by_name['value']._serialized_options = b'\272H\003\310\001\001'
  _NODETAINT.fields_by_name['effect']._options = None
  _NODETAINT.fields_by_name['effect']._serialized_options = b'\272H\003\310\001\001'
  _NODEGROUPDEPLOYMENTSTRATEGY.fields_by_name['drain_timeout']._options = None
  _NODEGROUPDEPLOYMENTSTRATEGY.fields_by_name['drain_timeout']._serialized_options = b'\272H\005\252\001\0022\000'
  _PERCENTORCOUNT.oneofs_by_name['value']._options = None
  _PERCENTORCOUNT.oneofs_by_name['value']._serialized_options = b'\272H\002\010\001'
  _PERCENTORCOUNT.fields_by_name['percent']._options = None
  _PERCENTORCOUNT.fields_by_name['percent']._serialized_options = b'\272H\006\"\004\030d(\000'
  _PERCENTORCOUNT.fields_by_name['count']._options = None
  _PERCENTORCOUNT.fields_by_name['count']._serialized_options = b'\272H\004\"\002(\000'
  _NODEAUTOREPAIRCONDITION.oneofs_by_name['trigger']._options = None
  _NODEAUTOREPAIRCONDITION.oneofs_by_name['trigger']._serialized_options = b'\272H\002\010\001'
  _NODEAUTOREPAIRCONDITION.fields_by_name['type']._options = None
  _NODEAUTOREPAIRCONDITION.fields_by_name['type']._serialized_options = b'\272H\003\310\001\001'
  _globals['_CONDITIONSTATUS']._serialized_start=4049
  _globals['_CONDITIONSTATUS']._serialized_end=4134
  _globals['_NODEGROUP']._serialized_start=251
  _globals['_NODEGROUP']._serialized_end=434
  _globals['_NODEGROUPSPEC']._serialized_start=437
  _globals['_NODEGROUPSPEC']._serialized_end=859
  _globals['_NODETEMPLATE']._serialized_start=862
  _globals['_NODETEMPLATE']._serialized_end=1620
  _globals['_NODEMETADATATEMPLATE']._serialized_start=1623
  _globals['_NODEMETADATATEMPLATE']._serialized_end=1788
  _globals['_NODEMETADATATEMPLATE_LABELSENTRY']._serialized_start=1731
  _globals['_NODEMETADATATEMPLATE_LABELSENTRY']._serialized_end=1788
  _globals['_GPUSETTINGS']._serialized_start=1790
  _globals['_GPUSETTINGS']._serialized_end=1850
  _globals['_GPUCLUSTERSPEC']._serialized_start=1852
  _globals['_GPUCLUSTERSPEC']._serialized_end=1884
  _globals['_NETWORKINTERFACETEMPLATE']._serialized_start=1887
  _globals['_NETWORKINTERFACETEMPLATE']._serialized_end=2031
  _globals['_PUBLICIPADDRESS']._serialized_start=2033
  _globals['_PUBLICIPADDRESS']._serialized_end=2050
  _globals['_ATTACHEDFILESYSTEMSPEC']._serialized_start=2053
  _globals['_ATTACHEDFILESYSTEMSPEC']._serialized_end=2370
  _globals['_ATTACHEDFILESYSTEMSPEC_ATTACHMODE']._serialized_start=2295
  _globals['_ATTACHEDFILESYSTEMSPEC_ATTACHMODE']._serialized_end=2355
  _globals['_EXISTINGFILESYSTEM']._serialized_start=2372
  _globals['_EXISTINGFILESYSTEM']._serialized_end=2416
  _globals['_NODEGROUPAUTOSCALINGSPEC']._serialized_start=2419
  _globals['_NODEGROUPAUTOSCALINGSPEC']._serialized_end=2690
  _globals['_NODETAINT']._serialized_start=2693
  _globals['_NODETAINT']._serialized_end=2917
  _globals['_NODETAINT_EFFECT']._serialized_start=2828
  _globals['_NODETAINT_EFFECT']._serialized_end=2917
  _globals['_NODEGROUPDEPLOYMENTSTRATEGY']._serialized_start=2920
  _globals['_NODEGROUPDEPLOYMENTSTRATEGY']._serialized_end=3157
  _globals['_PERCENTORCOUNT']._serialized_start=3159
  _globals['_PERCENTORCOUNT']._serialized_end=3263
  _globals['_NODEGROUPAUTOREPAIRSPEC']._serialized_start=3265
  _globals['_NODEGROUPAUTOREPAIRSPEC']._serialized_end=3363
  _globals['_NODEAUTOREPAIRCONDITION']._serialized_start=3366
  _globals['_NODEAUTOREPAIRCONDITION']._serialized_end=3579
  _globals['_PREEMPTIBLESPEC']._serialized_start=3581
  _globals['_PREEMPTIBLESPEC']._serialized_end=3598
  _globals['_NODEGROUPSTATUS']._serialized_start=3601
  _globals['_NODEGROUPSTATUS']._serialized_end=4047
  _globals['_NODEGROUPSTATUS_STATE']._serialized_start=3972
  _globals['_NODEGROUPSTATUS_STATE']._serialized_end=4047
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/federation_certificate_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.iam.v1 import federation_certificate_pb2 as nebius_dot_iam_dot_v1_dot_federation__certificate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2nebius/iam/v1/federation_certificate_service.proto\x12\rnebius.iam.v1\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a*nebius/iam/v1/federation_certificate.proto\"\xa2\x01\n\"CreateFederationCertificateRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12<\n\x04spec\x18\x02 \x01(\x0b\x32(.nebius.iam.v1.FederationCertificateSpecR\x04spec\"1\n\x1fGetFederationCertificateRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\x8f\x01\n,ListFederationCertificateByFederationRequest\x12#\n\rfederation_id\x18\x01 \x01(\tR\x0c\x66\x65\x64\x65rationId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"\xa2\x01\n\"UpdateFederationCertificateRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12<\n\x04spec\x18\x02 \x01(\x0b\x32(.nebius.iam.v1.FederationCertificateSpecR\x04spec\"\x9a\x01\n&UpdateBulkFederationCertificateRequest\x12#\n\rfederation_id\x18\x01 \x01(\tR\x0c\x66\x65\x64\x65rationId\x12K\n\x07updates\x18\x02 \x03(\x0b\x32\x31.nebius.iam.v1.UpdateFederationCertificateRequestR\x07updates\"4\n\"DeleteFederationCertificateRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\x87\x01\n!ListFederationCertificateResponse\x12:\n\x05items\x18\x01 \x03(\x0b\x32$.nebius.iam.v1.FederationCertificateR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xfb\x04\n\x1c\x46\x65\x64\x65rationCertificateService\x12X\n\x06\x43reate\x12\x31.nebius.iam.v1.CreateFederationCertificateRequest\x1a\x1b.nebius.common.v1.Operation\x12[\n\x03Get\x12..nebius.iam.v1.GetFederationCertificateRequest\x1a$.nebius.iam.v1.FederationCertificate\x12\x81\x01\n\x10ListByFederation\x12;.nebius.iam.v1.ListFederationCertificateByFederationRequest\x1a\x30.nebius.iam.v1.ListFederationCertificateResponse\x12X\n\x06Update\x12\x31.nebius.iam.v1.UpdateFederationCertificateRequest\x1a\x1b.nebius.common.v1.Operation\x12`\n\nUpdateBulk\x12\x35.nebius.iam.v1.UpdateBulkFederationCertificateRequest\x1a\x1b.nebius.common.v1.Operation\x12X\n\x06\x44\x65lete\x12\x31.nebius.iam.v1.DeleteFederationCertificateRequest\x1a\x1b.nebius.common.v1.Operation\x1a\n\xbaJ\x07\x63pl.iamBh\n\x14\x61i.nebius.pub.iam.v1B!FederationCertificateServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.federation_certificate_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B!FederationCertificateServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _FEDERATIONCERTIFICATESERVICE._options = None
  _FEDERATIONCERTIFICATESERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_CREATEFEDERATIONCERTIFICATEREQUEST']._serialized_start=207
  _globals['_CREATEFEDERATIONCERTIFICATEREQUEST']._serialized_end=369
  _globals['_GETFEDERATIONCERTIFICATEREQUEST']._serialized_start=371
  _globals['_GETFEDERATIONCERTIFICATEREQUEST']._serialized_end=420
  _globals['_LISTFEDERATIONCERTIFICATEBYFEDERATIONREQUEST']._serialized_start=423
  _globals['_LISTFEDERATIONCERTIFICATEBYFEDERATIONREQUEST']._serialized_end=566
  _globals['_UPDATEFEDERATIONCERTIFICATEREQUEST']._serialized_start=569
  _globals['_UPDATEFEDERATIONCERTIFICATEREQUEST']._serialized_end=731
  _globals['_UPDATEBULKFEDERATIONCERTIFICATEREQUEST']._serialized_start=734
  _globals['_UPDATEBULKFEDERATIONCERTIFICATEREQUEST']._serialized_end=888
  _globals['_DELETEFEDERATIONCERTIFICATEREQUEST']._serialized_start=890
  _globals['_DELETEFEDERATIONCERTIFICATEREQUEST']._serialized_end=942
  _globals['_LISTFEDERATIONCERTIFICATERESPONSE']._serialized_start=945
  _globals['_LISTFEDERATIONCERTIFICATERESPONSE']._serialized_end=1080
  _globals['_FEDERATIONCERTIFICATESERVICE']._serialized_start=1083
  _globals['_FEDERATIONCERTIFICATESERVICE']._serialized_end=1718
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/common/v1alpha1/operation_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius.common.v1alpha1 import operation_pb2 as nebius_dot_common_dot_v1alpha1_dot_operation__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.nebius/common/v1alpha1/operation_service.proto\x12\x16nebius.common.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a&nebius/common/v1alpha1/operation.proto\x1a\x18nebius/annotations.proto\"J\n\x13GetOperationRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id:\x1b\x18\x01\xd2J\x16\x12\x14migrate to common/v1\"\xb1\x01\n\x15ListOperationsRequest\x12\'\n\x0bresource_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\nresourceId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x16\n\x06\x66ilter\x18\x04 \x01(\tR\x06\x66ilter:\x1b\x18\x01\xd2J\x16\x12\x14migrate to common/v1\"\xa0\x01\n\x16ListOperationsResponse\x12\x41\n\noperations\x18\x01 \x03(\x0b\x32!.nebius.common.v1alpha1.OperationR\noperations\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken:\x1b\x18\x01\xd2J\x16\x12\x14migrate to common/v1\"\xb5\x01\n\x1dListOperationsByParentRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x16\n\x06\x66ilter\x18\x04 \x01(\tR\x06\x66ilter:\x1b\x18\x01\xd2J\x16\x12\x14migrate to common/v12\xee\x01\n\x10OperationService\x12U\n\x03Get\x12+.nebius.common.v1alpha1.GetOperationRequest\x1a!.nebius.common.v1alpha1.Operation\x12\x65\n\x04List\x12-.nebius.common.v1alpha1.ListOperationsRequest\x1a..nebius.common.v1alpha1.ListOperationsResponse\x1a\x1c\x88\x02\x01\xd2J\x16\x12\x14migrate to common/v1B\x8a\x01\n\x1d\x61i.nebius.pub.common.v1alpha1B\x15OperationServiceProtoP\x01Z4github.com/nebius/gosdk/proto/nebius/common/v1alpha1\xb8\x01\x01\xd2J\x16\x12\x14migrate to common/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.common.v1alpha1.operation_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035ai.nebius.pub.common.v1alpha1B\025OperationServiceProtoP\001Z4github.com/nebius/gosdk/proto/nebius/common/v1alpha1\270\001\001\322J\026\022\024migrate to common/v1'
  _GETOPERATIONREQUEST.fields_by_name['id']._options = None
  _GETOPERATIONREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _GETOPERATIONREQUEST._options = None
  _GETOPERATIONREQUEST._serialized_options = b'\030\001\322J\026\022\024migrate to common/v1'
  _LISTOPERATIONSREQUEST.fields_by_name['resource_id']._options = None
  _LISTOPERATIONSREQUEST.fields_by_name['resource_id']._serialized_options = b'\272H\003\310\001\001'
  _LISTOPERATIONSREQUEST._options = None
  _LISTOPERATIONSREQUEST._serialized_options = b'\030\001\322J\026\022\024migrate to common/v1'
  _LISTOPERATIONSRESPONSE._options = None
  _LISTOPERATIONSRESPONSE._serialized_options = b'\030\001\322J\026\022\024migrate to common/v1'
  _LISTOPERATIONSBYPARENTREQUEST.fields_by_name['parent_id']._options = None
  _LISTOPERATIONSBYPARENTREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _LISTOPERATIONSBYPARENTREQUEST._options = None
  _LISTOPERATIONSBYPARENTREQUEST._serialized_options = b'\030\001\322J\026\022\024migrate to common/v1'
  _OPERATIONSERVICE._options = None
  _OPERATIONSERVICE._serialized_options = b'\210\002\001\322J\026\022\024migrate to common/v1'
  _globals['_GETOPERATIONREQUEST']._serialized_start=169
  _globals['_GETOPERATIONREQUEST']._serialized_end=243
  _globals['_LISTOPERATIONSREQUEST']._serialized_start=246
  _globals['_LISTOPERATIONSREQUEST']._serialized_end=423
  _globals['_LISTOPERATIONSRESPONSE']._serialized_start=426
  _globals['_LISTOPERATIONSRESPONSE']._serialized_end=586
  _globals['_LISTOPERATIONSBYPARENTREQUEST']._serialized_start=589
  _globals['_LISTOPERATIONSBYPARENTREQUEST']._serialized_end=770
  _globals['_OPERATIONSERVICE']._serialized_start=773
  _globals['_OPERATIONSERVICE']._serialized_end=1011
# @@protoc_insertion_point(module_scope)



"""Token-based authenticators and providers.

This module provides a small concrete implementation of the
authorization interfaces that fetches bearer tokens from a token provider
and injects them into request metadata.
"""

from nebius.base.metadata import Metadata

from ..token import token
from .authorization import Authenticator, Provider

HEADER = "authorization"
"""The metadata header used for the bearer token (``"authorization"``).
"""


class TokenAuthenticator(Authenticator):
    """Per-request authenticator that injects a bearer token.

    The authenticator uses a :class:`token.Receiver` to fetch a token which
    is then written into the request metadata under the header named by
    :data:`HEADER` as ``Bearer <token>``. The instance holds a reference to
    the receiver and delegates retry decisions to the receiver via
    :meth:`token.Receiver.can_retry`.

    :param receiver: Token receiver used to fetch tokens for each request.
    :type receiver: :class:`token.Receiver`
    """

    def __init__(self, receiver: token.Receiver) -> None:
        """Create a token authenticator."""
        super().__init__()
        self._receiver = receiver

    async def authenticate(
        self,
        metadata: Metadata,
        timeout: float | None = None,
        options: dict[str, str] | None = None,
    ) -> None:
        """Fetch a token from the receiver and set the ``authorization`` metadata
        header.

        The header is replaced (any existing value is removed) with a
        ``Bearer <token>`` value fetched from the receiver.

        :param metadata: Metadata mapping sent with the request; mutated in
            place to include the Authorization header.
        :type metadata: :class:`nebius.base.metadata.Metadata`
        :param timeout: Optional timeout for the token fetch operation.
        :type timeout: optional `float`
        :param options: Optional provider/request options passed through.
        :type options: optional ``dict[str, str]``
        """
        tok = await self._receiver.fetch(timeout=timeout, options=options)
        del metadata[HEADER]
        metadata.add(HEADER, f"Bearer {tok.token}")

    def can_retry(
        self,
        err: Exception,
        options: dict[str, str] | None = None,
    ) -> bool:
        """Delegate retry decision to the underlying token receiver.

        :param err: The exception that caused the request/authentication
            to fail.
        :type err: :class:`Exception`
        :param options: Optional provider/request options.
        :type options: optional ``dict[str, str]``
        :returns: `True` when the receiver indicates the operation should be
            retried (for example token refresh may succeed); `False` otherwise.
        :rtype: bool
        """
        return self._receiver.can_retry(err, options)


class TokenProvider(Provider):
    """Provider that builds :class:`TokenAuthenticator` instances.

    The provider owns a :class:`token.Bearer` and returns a fresh
    :class:`TokenAuthenticator` for each call to :meth:`authenticator`.

    :param token_provider: The bearer token provider used to obtain
        receivers for per-request authenticators.
    :type token_provider: :class:`token.Bearer`

    Example
    -------

    Construct a bearer-backed provider and (illustratively) pass it to the
    SDK via the credentials parameter::

        from nebius.aio.token.static import EnvBearer
        from nebius.aio.authorization.token import TokenProvider
        from nebius.sdk import SDK

        bearer = EnvBearer("NEBIUS_IAM_TOKEN")
        provider = TokenProvider(bearer)
        sdk = SDK(credentials=provider)  # illustrative only

    """

    def __init__(self, token_provider: token.Bearer) -> None:
        """Create a token provider."""
        super().__init__()
        self._provider = token_provider

    def authenticator(self) -> Authenticator:
        """Return a fresh :class:`TokenAuthenticator` for a request.

        :returns: A token-based authenticator instance.
        :rtype: :class:`TokenAuthenticator`
        """
        return TokenAuthenticator(self._provider.receiver())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/access.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1anebius/iam/v1/access.proto\x12\rnebius.iam.v1\"\xd9\x02\n\x07\x41\x63\x63ount\x12G\n\x0cuser_account\x18\x01 \x01(\x0b\x32\".nebius.iam.v1.Account.UserAccountH\x00R\x0buserAccount\x12P\n\x0fservice_account\x18\x02 \x01(\x0b\x32%.nebius.iam.v1.Account.ServiceAccountH\x00R\x0eserviceAccount\x12V\n\x11\x61nonymous_account\x18\x03 \x01(\x0b\x32\'.nebius.iam.v1.Account.AnonymousAccountH\x00R\x10\x61nonymousAccount\x1a\x1d\n\x0bUserAccount\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x1a \n\x0eServiceAccount\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x1a\x12\n\x10\x41nonymousAccountB\x06\n\x04typeBR\n\x14\x61i.nebius.pub.iam.v1B\x0b\x41\x63\x63\x65ssProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.access_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\013AccessProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _globals['_ACCOUNT']._serialized_start=46
  _globals['_ACCOUNT']._serialized_end=391
  _globals['_ACCOUNT_USERACCOUNT']._serialized_start=300
  _globals['_ACCOUNT_USERACCOUNT']._serialized_end=329
  _globals['_ACCOUNT_SERVICEACCOUNT']._serialized_start=331
  _globals['_ACCOUNT_SERVICEACCOUNT']._serialized_end=363
  _globals['_ACCOUNT_ANONYMOUSACCOUNT']._serialized_start=365
  _globals['_ACCOUNT_ANONYMOUSACCOUNT']._serialized_end=383
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Project(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: ProjectSpec
    status: ProjectStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[ProjectSpec, _Mapping]] = ..., status: _Optional[_Union[ProjectStatus, _Mapping]] = ...) -> None: ...

class ProjectSpec(_message.Message):
    __slots__ = ["region"]
    REGION_FIELD_NUMBER: _ClassVar[int]
    region: str
    def __init__(self, region: _Optional[str] = ...) -> None: ...

class ProjectStatus(_message.Message):
    __slots__ = ["project_state"]
    class ProjectState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[ProjectStatus.ProjectState]
        CREATING: _ClassVar[ProjectStatus.ProjectState]
        ACTIVE: _ClassVar[ProjectStatus.ProjectState]
        PURGING: _ClassVar[ProjectStatus.ProjectState]
        CREATED: _ClassVar[ProjectStatus.ProjectState]
        ACTIVATING: _ClassVar[ProjectStatus.ProjectState]
        PARKING: _ClassVar[ProjectStatus.ProjectState]
        PARKED: _ClassVar[ProjectStatus.ProjectState]
    STATE_UNSPECIFIED: ProjectStatus.ProjectState
    CREATING: ProjectStatus.ProjectState
    ACTIVE: ProjectStatus.ProjectState
    PURGING: ProjectStatus.ProjectState
    CREATED: ProjectStatus.ProjectState
    ACTIVATING: ProjectStatus.ProjectState
    PARKING: ProjectStatus.ProjectState
    PARKED: ProjectStatus.ProjectState
    PROJECT_STATE_FIELD_NUMBER: _ClassVar[int]
    project_state: ProjectStatus.ProjectState
    def __init__(self, project_state: _Optional[_Union[ProjectStatus.ProjectState, str]] = ...) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius.common.v1 import operation_pb2 as _operation_pb2
from nebius.api.nebius.storage.v1 import bucket_pb2 as _bucket_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GetBucketRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class GetBucketByNameRequest(_message.Message):
    __slots__ = ["parent_id", "name"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    name: str
    def __init__(self, parent_id: _Optional[str] = ..., name: _Optional[str] = ...) -> None: ...

class CreateBucketRequest(_message.Message):
    __slots__ = ["metadata", "spec"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: _bucket_pb2.BucketSpec
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[_bucket_pb2.BucketSpec, _Mapping]] = ...) -> None: ...

class UpdateBucketRequest(_message.Message):
    __slots__ = ["metadata", "spec"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: _bucket_pb2.BucketSpec
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[_bucket_pb2.BucketSpec, _Mapping]] = ...) -> None: ...

class DeleteBucketRequest(_message.Message):
    __slots__ = ["id", "purge_at", "ttl"]
    ID_FIELD_NUMBER: _ClassVar[int]
    PURGE_AT_FIELD_NUMBER: _ClassVar[int]
    TTL_FIELD_NUMBER: _ClassVar[int]
    id: str
    purge_at: _timestamp_pb2.Timestamp
    ttl: _duration_pb2.Duration
    def __init__(self, id: _Optional[str] = ..., purge_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., ttl: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class PurgeBucketRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class UndeleteBucketRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class ListBucketsRequest(_message.Message):
    __slots__ = ["parent_id", "page_size", "page_token", "filter"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    page_size: int
    page_token: str
    filter: str
    def __init__(self, parent_id: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., filter: _Optional[str] = ...) -> None: ...

class ListBucketsResponse(_message.Message):
    __slots__ = ["items", "next_page_token"]
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[_bucket_pb2.Bucket]
    next_page_token: str
    def __init__(self, items: _Optional[_Iterable[_Union[_bucket_pb2.Bucket, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/compute/v1/maintenance_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.compute.v1 import maintenance_event_pb2 as nebius_dot_compute_dot_v1_dot_maintenance__event__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+nebius/compute/v1/maintenance_service.proto\x12\x11nebius.compute.v1\x1a\x18nebius/annotations.proto\x1a)nebius/compute/v1/maintenance_event.proto\"G\n$GetMaintenanceEventByInstanceRequest\x12\x1f\n\x0binstance_id\x18\x01 \x01(\tR\ninstanceId\"w\n\x1cListMaintenanceEventsRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12\x1b\n\tpage_size\x18\x03 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x04 \x01(\tR\tpageToken\"\x82\x01\n\x1dListMaintenanceEventsResponse\x12\x39\n\x05items\x18\x01 \x03(\x0b\x32#.nebius.compute.v1.MaintenanceEventR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\x80\x02\n\x12MaintenanceService\x12m\n\rGetByInstance\x12\x37.nebius.compute.v1.GetMaintenanceEventByInstanceRequest\x1a#.nebius.compute.v1.MaintenanceEvent\x12o\n\nListActive\x12/.nebius.compute.v1.ListMaintenanceEventsRequest\x1a\x30.nebius.compute.v1.ListMaintenanceEventsResponse\x1a\n\xbaJ\x07\x63omputeBf\n\x18\x61i.nebius.pub.compute.v1B\x17MaintenanceServiceProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/compute/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.compute.v1.maintenance_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.compute.v1B\027MaintenanceServiceProtoP\001Z/github.com/nebius/gosdk/proto/nebius/compute/v1'
  _MAINTENANCESERVICE._options = None
  _MAINTENANCESERVICE._serialized_options = b'\272J\007compute'
  _globals['_GETMAINTENANCEEVENTBYINSTANCEREQUEST']._serialized_start=135
  _globals['_GETMAINTENANCEEVENTBYINSTANCEREQUEST']._serialized_end=206
  _globals['_LISTMAINTENANCEEVENTSREQUEST']._serialized_start=208
  _globals['_LISTMAINTENANCEEVENTSREQUEST']._serialized_end=327
  _globals['_LISTMAINTENANCEEVENTSRESPONSE']._serialized_start=330
  _globals['_LISTMAINTENANCEEVENTSRESPONSE']._serialized_end=460
  _globals['_MAINTENANCESERVICE']._serialized_start=463
  _globals['_MAINTENANCESERVICE']._serialized_end=719
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/vpc/v1/network_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.vpc.v1 import network_pb2 as nebius_dot_vpc_dot_v1_dot_network__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#nebius/vpc/v1/network_service.proto\x12\rnebius.vpc.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a\x1bnebius/vpc/v1/network.proto\"+\n\x11GetNetworkRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"Z\n\x17GetNetworkByNameRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"v\n\x13ListNetworksRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\"l\n\x14ListNetworksResponse\x12,\n\x05items\x18\x01 \x03(\x0b\x32\x16.nebius.vpc.v1.NetworkR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\xd3\x02\n\x14\x43reateNetworkRequest\x12\x8a\x02\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\xc9\x01\xbaH\xc5\x01\xba\x01\xbe\x01\n\rmetadata_name\x12s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\x1a\x38this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\xc8\x01\x01R\x08metadata\x12.\n\x04spec\x18\x02 \x01(\x0b\x32\x1a.nebius.vpc.v1.NetworkSpecR\x04spec\"\xaa\x02\n\x1b\x43reateDefaultNetworkRequest\x12\x8a\x02\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\xc9\x01\xbaH\xc5\x01\xba\x01\xbe\x01\n\rmetadata_name\x12s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\x1a\x38this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\xc8\x01\x01R\x08metadata\"\xd0\x02\n\x14UpdateNetworkRequest\x12\x87\x02\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\xc6\x01\xbaH\xc2\x01\xba\x01\xbe\x01\n\rmetadata_name\x12s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\x1a\x38this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')R\x08metadata\x12.\n\x04spec\x18\x02 \x01(\x0b\x32\x1a.nebius.vpc.v1.NetworkSpecR\x04spec\".\n\x14\x44\x65leteNetworkRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id2\xb5\x04\n\x0eNetworkService\x12?\n\x03Get\x12 .nebius.vpc.v1.GetNetworkRequest\x1a\x16.nebius.vpc.v1.Network\x12K\n\tGetByName\x12&.nebius.vpc.v1.GetNetworkByNameRequest\x1a\x16.nebius.vpc.v1.Network\x12O\n\x04List\x12\".nebius.vpc.v1.ListNetworksRequest\x1a#.nebius.vpc.v1.ListNetworksResponse\x12J\n\x06\x43reate\x12#.nebius.vpc.v1.CreateNetworkRequest\x1a\x1b.nebius.common.v1.Operation\x12X\n\rCreateDefault\x12*.nebius.vpc.v1.CreateDefaultNetworkRequest\x1a\x1b.nebius.common.v1.Operation\x12J\n\x06Update\x12#.nebius.vpc.v1.UpdateNetworkRequest\x1a\x1b.nebius.common.v1.Operation\x12J\n\x06\x44\x65lete\x12#.nebius.vpc.v1.DeleteNetworkRequest\x1a\x1b.nebius.common.v1.Operation\x1a\x06\xbaJ\x03vpcBZ\n\x14\x61i.nebius.pub.vpc.v1B\x13NetworkServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/vpc/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.vpc.v1.network_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.vpc.v1B\023NetworkServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/vpc/v1'
  _GETNETWORKREQUEST.fields_by_name['id']._options = None
  _GETNETWORKREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _GETNETWORKBYNAMEREQUEST.fields_by_name['parent_id']._options = None
  _GETNETWORKBYNAMEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _GETNETWORKBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETNETWORKBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _LISTNETWORKSREQUEST.fields_by_name['parent_id']._options = None
  _LISTNETWORKSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _CREATENETWORKREQUEST.fields_by_name['metadata']._options = None
  _CREATENETWORKREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\305\001\272\001\276\001\n\rmetadata_name\022s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\0328this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\310\001\001'
  _CREATEDEFAULTNETWORKREQUEST.fields_by_name['metadata']._options = None
  _CREATEDEFAULTNETWORKREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\305\001\272\001\276\001\n\rmetadata_name\022s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\0328this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')\310\001\001'
  _UPDATENETWORKREQUEST.fields_by_name['metadata']._options = None
  _UPDATENETWORKREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\302\001\272\001\276\001\n\rmetadata_name\022s\'name\' must start with a letter or digit, allow \'-\', \'_\', \'.\', \'/\', and have a length between 2 and 255 characters.\0328this.name.matches(\'^[a-zA-Z0-9][-_./a-zA-Z0-9]{1,254}$\')'
  _DELETENETWORKREQUEST.fields_by_name['id']._options = None
  _DELETENETWORKREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _NETWORKSERVICE._options = None
  _NETWORKSERVICE._serialized_options = b'\272J\003vpc'
  _globals['_GETNETWORKREQUEST']._serialized_start=205
  _globals['_GETNETWORKREQUEST']._serialized_end=248
  _globals['_GETNETWORKBYNAMEREQUEST']._serialized_start=250
  _globals['_GETNETWORKBYNAMEREQUEST']._serialized_end=340
  _globals['_LISTNETWORKSREQUEST']._serialized_start=342
  _globals['_LISTNETWORKSREQUEST']._serialized_end=460
  _globals['_LISTNETWORKSRESPONSE']._serialized_start=462
  _globals['_LISTNETWORKSRESPONSE']._serialized_end=570
  _globals['_CREATENETWORKREQUEST']._serialized_start=573
  _globals['_CREATENETWORKREQUEST']._serialized_end=912
  _globals['_CREATEDEFAULTNETWORKREQUEST']._serialized_start=915
  _globals['_CREATEDEFAULTNETWORKREQUEST']._serialized_end=1213
  _globals['_UPDATENETWORKREQUEST']._serialized_start=1216
  _globals['_UPDATENETWORKREQUEST']._serialized_end=1552
  _globals['_DELETENETWORKREQUEST']._serialized_start=1554
  _globals['_DELETENETWORKREQUEST']._serialized_end=1600
  _globals['_NETWORKSERVICE']._serialized_start=1603
  _globals['_NETWORKSERVICE']._serialized_end=2168
# @@protoc_insertion_point(module_scope)



"""Asynchronous bearer token abstractions and the token representation.

This module provides the core abstractions used by the SDK for representing
bearer tokens and for supplying per-request token receivers.

Key classes
-----------

- :class:`Token` -- simple immutable representation of an access token and an
    optional expiration time.
- :class:`Receiver` -- asynchronous primitive that can fetch a single
    :class:`Token` (used per-request by the authentication layer).
- :class:`Bearer` -- provider of :class:`Receiver` instances. Bearers are
    composed to add features like naming, renewal and caching.

Examples
--------
Create a simple token and serialize it::

        from nebius.aio.token.token import Token
        from datetime import datetime, timezone, timedelta

        tok = Token(
                "s3cr3t",
                expiration=datetime.now(timezone.utc) + timedelta(hours=1),
        )
        print(str(tok))
        data = tok.to_dict()

Restore the token from the serialized form::

        tok2 = Token.from_dict(data)
        assert tok == tok2

Using a static bearer/receiver (see :mod:`nebius.aio.token.static` for
concrete implementations) the SDK creates per-request receivers with
``bearer.receiver()`` and uses :meth:`Receiver.fetch` to obtain tokens.
"""

from abc import ABC, abstractmethod
from datetime import datetime, timezone
from typing import Any

from nebius.base.token_sanitizer import TokenSanitizer

sanitizer = TokenSanitizer.access_token_sanitizer()


class Token:
    """Representation of a bearer token with optional expiration.

    Behaviour notes
    ---------------
    - ``str(Token)`` returns a sanitized representation using
      :class:`nebius.base.token_sanitizer.TokenSanitizer` to avoid logging
      secrets in plaintext. When expiration is present, the ISO timestamp and
      a relative ``expires_in`` delta are included in the representation.
    - ``to_dict`` serializes the token into a mapping with ``token`` and
      ``expires_at`` keys, where ``expires_at`` is an integer POSIX timestamp
      (or ``None`` when no expiration was set).

    :param token: The raw token string. May be empty for an "empty" token.
    :type token: `str`
    :param expiration: Optional UTC-aware expiration datetime. If provided,
        :meth:`is_expired` checks against this value.
    :type expiration: :class:`datetime.datetime` or `None`
    """

    def __init__(self, token: str, expiration: datetime | None = None) -> None:
        """Initialize a token with these token string and expiration"""
        self._tok = token
        self._exp = expiration

    def __str__(self) -> str:
        """Return a short, sanitized string representation.

        :returns: A one-line safe representation of the token for debugging.
        :rtype: `str`
        """
        if self.is_empty():
            return "Token(empty)"
        ret = ["Token("]
        ret.append(sanitizer.sanitize(self._tok))
        if self._exp is not None:
            ret.append(f", expiration={self._exp.isoformat()}")
            expires_in = self._exp - datetime.now(timezone.utc)
            ret.append(f", expires_in={expires_in}")
        else:
            ret.append(", expiration=None")
        ret.append(")")
        return "".join(ret)

    @classmethod
    def empty(cls) -> "Token":
        """Create an empty token.

        :returns: A :class:`Token` instance representing the absence of a
            credential (empty token string).
        :rtype: :class:`Token`
        """
        return cls(token="")

    @property
    def token(self) -> str:
        """The raw token string.

        :rtype: `str`
        """
        return self._tok

    @property
    def expiration(self) -> datetime | None:
        """The UTC-aware expiration datetime, or ``None`` if not set.

        :rtype: :class:`datetime.datetime` or `None`
        """
        return self._exp

    def is_empty(self) -> bool:
        """Return `True` when the token string is empty.

        :returns: `True` when this token represents no credential.
        """
        return self._tok == ""

    def is_expired(self) -> bool:
        """Return `True` when the token is expired.

        When ``expiration`` is `None`, tokens are considered non-expiring and
        this method returns `False`.

        :returns: `True` if the current UTC time is equal to or after the
            expiration instant.
        :rtype: bool
        """
        if self._exp is None:
            return False
        return datetime.now(timezone.utc) >= self._exp

    def to_dict(self) -> dict[str, Any]:
        """Serialize the token to a mapping.

        The mapping contains ``token`` and ``expires_at``. ``expires_at`` is
        an integer POSIX timestamp when an expiration is set, otherwise
        ``None``.

        :returns: A dictionary serializable to JSON.
        :rtype: `dict`
        """
        expires_at = int(self._exp.timestamp()) if self._exp is not None else 0
        data: dict[str, Any] = {
            "token": self._tok,
            "expires_at": expires_at if self._exp is not None else None,
        }
        return data

    def __eq__(self, value: object) -> bool:
        """Equality comparison.

        Two tokens are equal when both their token strings and expiration
        datetimes are equal.

        :param value: Other object to compare.
        :returns: `True` when equal, otherwise `False`.
        """
        if not isinstance(value, Token):
            return NotImplemented
        return self._tok == value._tok and self._exp == value._exp

    @classmethod
    def from_dict(cls, data: Any) -> "Token":
        """Create a Token from a mapping produced by :meth:`to_dict`.

        :param data: Mapping with at least the ``token`` key and optional
            ``expires_at`` integer timestamp.
        :type data: :class:`typing.Any`
        :raises ValueError: When the input type is invalid or contains
            incompatible types.
        :returns: A :class:`Token` instance parsed from the mapping.
        :rtype: :class:`Token`
        """
        if not isinstance(data, dict):
            raise ValueError(
                f"Invalid format for Token: {type(data)} expected a dictionary.",
            )
        token = data.get("token", "")  # type: ignore[assignment,unused-ignore]
        if token is None:
            token = ""
        if not isinstance(token, str):
            raise ValueError(f"Invalid token format: {type(token)} expected a string.")  # type: ignore[assignment,unused-ignore]
        expires_at = data.get("expires_at", None)  # type: ignore[assignment,unused-ignore]
        if expires_at is None:
            return cls(token=token)
        if not isinstance(expires_at, int):
            raise ValueError(
                f"Invalid expires_at format: {type(expires_at)} expected an int."  # type: ignore[assignment,unused-ignore]
            )
        expiration = (
            datetime.fromtimestamp(expires_at, tz=timezone.utc) if expires_at else None
        )
        return cls(token=token, expiration=expiration)


class Receiver(ABC):
    """Abstract asynchronous token receiver interface.

    Implementations fetch bearer tokens from an external source (for
    example an OAuth token endpoint or a local cache) and expose a small API
    that the request layer uses to obtain tokens for per-request
    authentication.

    The receiver is responsible for fetching and refreshing the token for one request
    only, reflecting the authentication process, while the :class:`Bearer` class
    supports multiple requests and provides per-request receivers on demand.
    """

    _latest: Token | None
    """The most recently fetched token, or `None` if no token has been
    fetched yet. Implementations may update this to support inspection or
    caching by callers.
    """

    @abstractmethod
    async def _fetch(
        self, timeout: float | None = None, options: dict[str, str] | None = None
    ) -> Token:
        """Low-level asynchronous fetch implementation.

        Subclasses must implement this method to perform the actual token
        retrieval. This method is intentionally prefixed with an underscore to
        indicate it is the minimal primitive; callers should use
        :meth:`fetch` which records the result in ``_latest``.

        :param timeout: Optional timeout in seconds for the fetch operation.
        :type timeout: optional `float`
        :param options: Optional implementation-specific options forwarded by
            the request layer.
        :type options: optional ``dict[str, str]``
        :returns: A freshly fetched :class:`Token`.
        :rtype: :class:`Token`
        """
        raise NotImplementedError("Method not implemented!")

    @property
    def latest(self) -> Token | None:
        """Return the latest fetched token or ``None``.

        :rtype: :class:`Token` or None
        """
        return self._latest

    async def fetch(
        self, timeout: float | None = None, options: dict[str, str] | None = None
    ) -> Token:
        """Fetch a token and record it as the latest value.

        This method calls the concrete :meth:`_fetch` implementation and stores
        the result on :attr:`_latest` before returning it.

        :param timeout: Optional timeout in seconds forwarded to the fetch.
        :type timeout: optional `float`
        :param options: Optional implementation-specific options.
        :type options: optional ``dict[str, str]``
        :returns: The fetched :class:`Token`.
        :rtype: :class:`Token`
        """
        tok = await self._fetch(timeout=timeout, options=options)
        self._latest = tok
        return tok

    @abstractmethod
    def can_retry(
        self,
        err: Exception,
        options: dict[str, str] | None = None,
    ) -> bool:
        """Decide whether a failed authentication attempt should be retried.

        Implementations inspect ``err`` (for example gRPC status codes or
        specific exception types) and optional ``options`` to decide whether a
        fresh call to :meth:`fetch` is likely to succeed (for example after
        refreshing credentials).

        Default implementations should return `False`.

        :param err: The exception that triggered the retry decision.
        :type err: `Exception`
        :param options: Optional implementation-specific options.
        :type options: ``dict[str, str]`` or `None`
        :returns: `True` when a retry is advisable, otherwise `False`.
        """
        return False


class Bearer(ABC):
    """Abstract provider of :class:`Receiver` instances.

    A Bearer supplies receivers for per-request authenticators. Bearers may
    be composed (wrapping other Bearers) to add behaviour such as caching,
    refreshing, or naming.

    Example
    -------

    Implement a custom bearer::

        from nebius.sdk import SDK
        from nebius.aio.token.token import Bearer, Receiver, Token

        class MyBearer(Bearer):
            def receiver(self) -> Receiver:
                return MyReceiver()

        class MyReceiver(Receiver):
            async def _fetch(self, timeout=None, options=None) -> Token:
                return Token("my-token")

            def can_retry(self, err, options=None) -> bool:
                return False

        sdk = SDK(credentials=MyBearer())
    """

    @abstractmethod
    def receiver(self) -> Receiver:
        """Return a :class:`Receiver` to be used for a single request.

        Implementations should return a receiver that can safely be used by
        a single request/Authenticator instance. It may return a shared
        receiver or create a new one depending on the provider semantics.

        :returns: A fresh and unique token :class:`Receiver`.
        :rtype: :class:`Receiver`
        """
        raise NotImplementedError("Method not implemented!")

    @property
    def name(self) -> str | None:
        """Optional human-readable name for the bearer.

        This may be used in some wrapper providers to cache the results of
        the shared receiver, or to report diagnostics.

        This name should reflect the unique configuration of the bearer.

        If the bearer wraps another bearer, the default behaviour is to
        forward the name lookup to the wrapped instance.

        :rtype: `str` or `None`
        """
        if self.wrapped is not None:
            return self.wrapped.name
        return None

    @property
    def wrapped(self) -> "Bearer|None":
        """Return the wrapped bearer or `None` if not wrapping.

        Subclasses that decorate or compose another bearer should override this
        property to return the inner bearer.

        :rtype: :class:`Bearer` or `None`
        """
        return None

    async def close(self, grace: float | None = None) -> None:
        """Close the bearer and any wrapped resources.

        :param grace: Optional graceful shutdown timeout in seconds.
        :type grace: optional `float`
        """
        if self.wrapped is not None:
            await self.wrapped.close(grace=grace)
        return None


class NamedBearer(Bearer):
    """Simple bearer wrapper that attaches a static name to another bearer.

    This may be used in some wrapper providers to cache the results of
    the shared receiver, or to report diagnostics.

    This name should reflect the unique configuration of the underlying bearer.

    :param wrapped: The inner bearer to delegate to.
    :type wrapped: :class:`Bearer`
    :param name: The :meth:`name` that reflects the configuration of the underlying
        bearer.

    Example
    -------

    Wrap a custom bearer with a name and file cache::

        from nebius.sdk import SDK
        from nebius.aio.token.token import NamedBearer, Bearer, Receiver, Token
        from nebius.aio.token.file_cache.async_renewable_bearer import (
            AsynchronousRenewableFileCacheBearer
        )

        class SomeCustomHeavyLoadBearer(Bearer):
            def receiver(self) -> Receiver:
                return SomeReceiver()

        class SomeReceiver(Receiver):
            async def _fetch(self, timeout=None, options=None) -> Token:
                # Simulate heavy load token fetch
                return Token("heavy-token")

            def can_retry(self, err, options=None) -> bool:
                return False

        custom_bearer = SomeCustomHeavyLoadBearer()
        named_bearer = NamedBearer(custom_bearer, "heavy-load-bearer")
        cached_bearer = AsynchronousRenewableFileCacheBearer(named_bearer)

        sdk = SDK(credentials=cached_bearer)
    """

    def __init__(self, wrapped: Bearer, name: str) -> None:
        """Wrap the passed-in bearer, attaching a name to it.

        This name should reflect the unique configuration of the underlying bearer.
        """
        self._wrapped = wrapped
        self._name = name

    @property
    def wrapped(self) -> Bearer:
        """Return the wrapped bearer."""
        return self._wrapped

    @property
    def name(self) -> str | None:
        """Return the configured name for this bearer.

        :returns: Always returns a name.
        :rtype: `str`
        """
        return self._name

    def receiver(self) -> Receiver:
        """Delegate to the wrapped bearer to obtain a receiver.

        :returns: A :class:`Receiver` obtained from the wrapped bearer.
        """
        return self._wrapped.receiver()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/federated_credentials_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.iam.v1 import federated_credentials_pb2 as nebius_dot_iam_dot_v1_dot_federated__credentials__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1nebius/iam/v1/federated_credentials_service.proto\x12\rnebius.iam.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a)nebius/iam/v1/federated_credentials.proto\"8\n\x1eGetFederatedCredentialsRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"g\n$GetByNameFederatedCredentialsRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"\xb0\x01\n!CreateFederatedCredentialsRequest\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x43\n\x04spec\x18\x02 \x01(\x0b\x32\'.nebius.iam.v1.FederatedCredentialsSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\"\xa0\x01\n!UpdateFederatedCredentialsRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12;\n\x04spec\x18\x02 \x01(\x0b\x32\'.nebius.iam.v1.FederatedCredentialsSpecR\x04spec\"\x9a\x01\n\x1fListFederatedCredentialsRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x16\n\x06\x66ilter\x18\x04 \x01(\tR\x06\x66ilter\"\x85\x01\n ListFederatedCredentialsResponse\x12\x39\n\x05items\x18\x01 \x03(\x0b\x32#.nebius.iam.v1.FederatedCredentialsR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\";\n!DeleteFederatedCredentialsRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id2\xdf\x04\n\x1b\x46\x65\x64\x65ratedCredentialsService\x12Y\n\x03Get\x12-.nebius.iam.v1.GetFederatedCredentialsRequest\x1a#.nebius.iam.v1.FederatedCredentials\x12\x65\n\tGetByName\x12\x33.nebius.iam.v1.GetByNameFederatedCredentialsRequest\x1a#.nebius.iam.v1.FederatedCredentials\x12g\n\x04List\x12..nebius.iam.v1.ListFederatedCredentialsRequest\x1a/.nebius.iam.v1.ListFederatedCredentialsResponse\x12W\n\x06\x43reate\x12\x30.nebius.iam.v1.CreateFederatedCredentialsRequest\x1a\x1b.nebius.common.v1.Operation\x12W\n\x06Update\x12\x30.nebius.iam.v1.UpdateFederatedCredentialsRequest\x1a\x1b.nebius.common.v1.Operation\x12W\n\x06\x44\x65lete\x12\x30.nebius.iam.v1.DeleteFederatedCredentialsRequest\x1a\x1b.nebius.common.v1.Operation\x1a\n\xbaJ\x07\x63pl.iamBg\n\x14\x61i.nebius.pub.iam.v1B FederatedCredentialsServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.federated_credentials_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B FederatedCredentialsServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _GETFEDERATEDCREDENTIALSREQUEST.fields_by_name['id']._options = None
  _GETFEDERATEDCREDENTIALSREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _GETBYNAMEFEDERATEDCREDENTIALSREQUEST.fields_by_name['parent_id']._options = None
  _GETBYNAMEFEDERATEDCREDENTIALSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _GETBYNAMEFEDERATEDCREDENTIALSREQUEST.fields_by_name['name']._options = None
  _GETBYNAMEFEDERATEDCREDENTIALSREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _CREATEFEDERATEDCREDENTIALSREQUEST.fields_by_name['metadata']._options = None
  _CREATEFEDERATEDCREDENTIALSREQUEST.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _CREATEFEDERATEDCREDENTIALSREQUEST.fields_by_name['spec']._options = None
  _CREATEFEDERATEDCREDENTIALSREQUEST.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _LISTFEDERATEDCREDENTIALSREQUEST.fields_by_name['parent_id']._options = None
  _LISTFEDERATEDCREDENTIALSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _DELETEFEDERATEDCREDENTIALSREQUEST.fields_by_name['id']._options = None
  _DELETEFEDERATEDCREDENTIALSREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _FEDERATEDCREDENTIALSSERVICE._options = None
  _FEDERATEDCREDENTIALSSERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_GETFEDERATEDCREDENTIALSREQUEST']._serialized_start=233
  _globals['_GETFEDERATEDCREDENTIALSREQUEST']._serialized_end=289
  _globals['_GETBYNAMEFEDERATEDCREDENTIALSREQUEST']._serialized_start=291
  _globals['_GETBYNAMEFEDERATEDCREDENTIALSREQUEST']._serialized_end=394
  _globals['_CREATEFEDERATEDCREDENTIALSREQUEST']._serialized_start=397
  _globals['_CREATEFEDERATEDCREDENTIALSREQUEST']._serialized_end=573
  _globals['_UPDATEFEDERATEDCREDENTIALSREQUEST']._serialized_start=576
  _globals['_UPDATEFEDERATEDCREDENTIALSREQUEST']._serialized_end=736
  _globals['_LISTFEDERATEDCREDENTIALSREQUEST']._serialized_start=739
  _globals['_LISTFEDERATEDCREDENTIALSREQUEST']._serialized_end=893
  _globals['_LISTFEDERATEDCREDENTIALSRESPONSE']._serialized_start=896
  _globals['_LISTFEDERATEDCREDENTIALSRESPONSE']._serialized_end=1029
  _globals['_DELETEFEDERATEDCREDENTIALSREQUEST']._serialized_start=1031
  _globals['_DELETEFEDERATEDCREDENTIALSREQUEST']._serialized_end=1090
  _globals['_FEDERATEDCREDENTIALSSERVICE']._serialized_start=1093
  _globals['_FEDERATEDCREDENTIALSSERVICE']._serialized_end=1700
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class NetworkInterfaceSpec(_message.Message):
    __slots__ = ["subnet_id", "name", "ip_address", "public_ip_address", "aliases"]
    SUBNET_ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    IP_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_IP_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    ALIASES_FIELD_NUMBER: _ClassVar[int]
    subnet_id: str
    name: str
    ip_address: IPAddress
    public_ip_address: PublicIPAddress
    aliases: _containers.RepeatedCompositeFieldContainer[IPAlias]
    def __init__(self, subnet_id: _Optional[str] = ..., name: _Optional[str] = ..., ip_address: _Optional[_Union[IPAddress, _Mapping]] = ..., public_ip_address: _Optional[_Union[PublicIPAddress, _Mapping]] = ..., aliases: _Optional[_Iterable[_Union[IPAlias, _Mapping]]] = ...) -> None: ...

class IPAddress(_message.Message):
    __slots__ = ["allocation_id"]
    ALLOCATION_ID_FIELD_NUMBER: _ClassVar[int]
    allocation_id: str
    def __init__(self, allocation_id: _Optional[str] = ...) -> None: ...

class PublicIPAddress(_message.Message):
    __slots__ = ["allocation_id", "static"]
    ALLOCATION_ID_FIELD_NUMBER: _ClassVar[int]
    STATIC_FIELD_NUMBER: _ClassVar[int]
    allocation_id: str
    static: bool
    def __init__(self, allocation_id: _Optional[str] = ..., static: bool = ...) -> None: ...

class IPAlias(_message.Message):
    __slots__ = ["allocation_id"]
    ALLOCATION_ID_FIELD_NUMBER: _ClassVar[int]
    allocation_id: str
    def __init__(self, allocation_id: _Optional[str] = ...) -> None: ...

class NetworkInterfaceStatus(_message.Message):
    __slots__ = ["index", "name", "ip_address", "public_ip_address", "aliases", "mac_address", "fqdn"]
    INDEX_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    IP_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_IP_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    ALIASES_FIELD_NUMBER: _ClassVar[int]
    MAC_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    FQDN_FIELD_NUMBER: _ClassVar[int]
    index: int
    name: str
    ip_address: IPAddressStatus
    public_ip_address: PublicIPAddressStatus
    aliases: IPAliasesStatus
    mac_address: str
    fqdn: str
    def __init__(self, index: _Optional[int] = ..., name: _Optional[str] = ..., ip_address: _Optional[_Union[IPAddressStatus, _Mapping]] = ..., public_ip_address: _Optional[_Union[PublicIPAddressStatus, _Mapping]] = ..., aliases: _Optional[_Union[IPAliasesStatus, _Mapping]] = ..., mac_address: _Optional[str] = ..., fqdn: _Optional[str] = ...) -> None: ...

class IPAddressStatus(_message.Message):
    __slots__ = ["address", "allocation_id"]
    ADDRESS_FIELD_NUMBER: _ClassVar[int]
    ALLOCATION_ID_FIELD_NUMBER: _ClassVar[int]
    address: str
    allocation_id: str
    def __init__(self, address: _Optional[str] = ..., allocation_id: _Optional[str] = ...) -> None: ...

class PublicIPAddressStatus(_message.Message):
    __slots__ = ["address", "allocation_id", "static"]
    ADDRESS_FIELD_NUMBER: _ClassVar[int]
    ALLOCATION_ID_FIELD_NUMBER: _ClassVar[int]
    STATIC_FIELD_NUMBER: _ClassVar[int]
    address: str
    allocation_id: str
    static: bool
    def __init__(self, address: _Optional[str] = ..., allocation_id: _Optional[str] = ..., static: bool = ...) -> None: ...

class IPAliasesStatus(_message.Message):
    __slots__ = ["cidrs"]
    CIDRS_FIELD_NUMBER: _ClassVar[int]
    cidrs: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, cidrs: _Optional[_Iterable[str]] = ...) -> None: ...



import argparse
import os
from time import time  # type: ignore[unused-ignore]

from nebius.aio.service_error import RequestError
from nebius.aio.token.static import Bearer
from nebius.aio.token.token import Token
from nebius.api.nebius.common.v1 import ResourceMetadata
from nebius.api.nebius.storage.v1 import (
    BucketServiceClient,
    BucketSpec,
    CreateBucketRequest,
    DeleteBucketRequest,
    GetBucketRequest,
    VersioningPolicy,
)
from nebius.sdk import SDK

if __name__ == "__main__":
    import logging

    logging.basicConfig(level=logging.DEBUG)

    def sync_main() -> None:
        parser = argparse.ArgumentParser(
            "examples/basic_sync.py",
            description="basic example that creates and then deletes a bucket",
        )
        parser.add_argument(
            "project_id", help="project in which to create and delete " "a test bucket."
        )
        args = parser.parse_args()

        sdk = SDK(
            credentials=Bearer(
                Token(
                    os.environ.get("NEBIUS_IAM_TOKEN", ""),
                )
            ),
        )
        project_id: str = args.project_id

        service = BucketServiceClient(sdk)
        try:
            req = service.create(
                CreateBucketRequest(
                    metadata=ResourceMetadata(
                        parent_id=project_id,
                        name=f"test-pysdk-bucket-{time()}",
                    ),
                    spec=BucketSpec(
                        versioning_policy=VersioningPolicy.DISABLED,
                        max_size_bytes=4096,
                    ),
                )
            )
            status = req.current_status()
            print(status)
            ret = req.wait()
            mdi = req.initial_metadata_sync()
            mdt = req.trailing_metadata_sync()
            status = req.current_status()
            # or just do `ret: Operation = await service.Create(req)`
            print(ret)
            print(mdi, mdt, status)
            ret.sync_wait()
            print(ret)
            bucket = service.get(GetBucketRequest(id=ret.resource_id)).wait()
            print(bucket)
            service.delete(DeleteBucketRequest(id=bucket.metadata.id)).wait()
        except RequestError as e:
            print(e)
            raise
        sdk.sync_close()

    sync_main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/billing/v1alpha1/calculator.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius.compute.v1 import disk_service_pb2 as nebius_dot_compute_dot_v1_dot_disk__service__pb2
from nebius.api.nebius.compute.v1 import filesystem_service_pb2 as nebius_dot_compute_dot_v1_dot_filesystem__service__pb2
from nebius.api.nebius.compute.v1 import instance_service_pb2 as nebius_dot_compute_dot_v1_dot_instance__service__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(nebius/billing/v1alpha1/calculator.proto\x12\x17nebius.billing.v1alpha1\x1a$nebius/compute/v1/disk_service.proto\x1a*nebius/compute/v1/filesystem_service.proto\x1a(nebius/compute/v1/instance_service.proto\"\xfa\x04\n\x0cResourceSpec\x12^\n\x15\x63ompute_instance_spec\x18\x64 \x01(\x0b\x32(.nebius.compute.v1.CreateInstanceRequestH\x00R\x13\x63omputeInstanceSpec\x12k\n\x1c\x63ompute_instance_update_spec\x18\x65 \x01(\x0b\x32(.nebius.compute.v1.UpdateInstanceRequestH\x00R\x19\x63omputeInstanceUpdateSpec\x12R\n\x11\x63ompute_disk_spec\x18\x66 \x01(\x0b\x32$.nebius.compute.v1.CreateDiskRequestH\x00R\x0f\x63omputeDiskSpec\x12_\n\x18\x63ompute_disk_update_spec\x18g \x01(\x0b\x32$.nebius.compute.v1.UpdateDiskRequestH\x00R\x15\x63omputeDiskUpdateSpec\x12\x64\n\x17\x63ompute_filesystem_spec\x18h \x01(\x0b\x32*.nebius.compute.v1.CreateFilesystemRequestH\x00R\x15\x63omputeFilesystemSpec\x12q\n\x1e\x63ompute_filesystem_update_spec\x18i \x01(\x0b\x32*.nebius.compute.v1.UpdateFilesystemRequestH\x00R\x1b\x63omputeFilesystemUpdateSpecB\x0f\n\rresource_spec\"g\n\x11ResourceGroupCost\x12\x45\n\x07general\x18\n \x01(\x0b\x32).nebius.billing.v1alpha1.GeneralTotalCostH\x00R\x07generalB\x0b\n\tcost_type\"P\n\x10GeneralTotalCost\x12<\n\x05total\x18\x01 \x01(\x0b\x32&.nebius.billing.v1alpha1.CostBreakdownR\x05total\"S\n\x13GeneralResourceCost\x12<\n\x05total\x18\x01 \x01(\x0b\x32&.nebius.billing.v1alpha1.CostBreakdownR\x05total\"F\n\rCostBreakdown\x12\x12\n\x04\x63ost\x18\x02 \x01(\tR\x04\x63ost\x12!\n\x0c\x63ost_rounded\x18\x03 \x01(\tR\x0b\x63ostRoundedBj\n\x1e\x61i.nebius.pub.billing.v1alpha1B\x0f\x43\x61lculatorProtoP\x01Z5github.com/nebius/gosdk/proto/nebius/billing/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.billing.v1alpha1.calculator_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036ai.nebius.pub.billing.v1alpha1B\017CalculatorProtoP\001Z5github.com/nebius/gosdk/proto/nebius/billing/v1alpha1'
  _globals['_RESOURCESPEC']._serialized_start=194
  _globals['_RESOURCESPEC']._serialized_end=828
  _globals['_RESOURCEGROUPCOST']._serialized_start=830
  _globals['_RESOURCEGROUPCOST']._serialized_end=933
  _globals['_GENERALTOTALCOST']._serialized_start=935
  _globals['_GENERALTOTALCOST']._serialized_end=1015
  _globals['_GENERALRESOURCECOST']._serialized_start=1017
  _globals['_GENERALRESOURCECOST']._serialized_end=1100
  _globals['_COSTBREAKDOWN']._serialized_start=1102
  _globals['_COSTBREAKDOWN']._serialized_end=1172
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius.maintenance.v1alpha1 import maintenance_pb2 as _maintenance_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ListMaintenancesRequest(_message.Message):
    __slots__ = ["parent_id"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    def __init__(self, parent_id: _Optional[str] = ...) -> None: ...

class ListMaintenancesResponse(_message.Message):
    __slots__ = ["items"]
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[_maintenance_pb2.Maintenance]
    def __init__(self, items: _Optional[_Iterable[_Union[_maintenance_pb2.Maintenance, _Mapping]]] = ...) -> None: ...

class GetMaintenanceRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class UpdateMaintenanceRequest(_message.Message):
    __slots__ = ["metadata", "spec"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: _maintenance_pb2.MaintenanceSpec
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[_maintenance_pb2.MaintenanceSpec, _Mapping]] = ...) -> None: ...

class UpdateMaintenanceResponse(_message.Message):
    __slots__ = ["maintenance"]
    MAINTENANCE_FIELD_NUMBER: _ClassVar[int]
    maintenance: _maintenance_pb2.Maintenance
    def __init__(self, maintenance: _Optional[_Union[_maintenance_pb2.Maintenance, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/user_account.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n nebius/iam/v1/user_account.proto\x12\rnebius.iam.v1\x1a\x18nebius/annotations.proto\"~\n\x15UserAccountExternalId\x12@\n\x1a\x66\x65\x64\x65ration_user_account_id\x18\x01 \x01(\tB\x03\xc0J\x01R\x17\x66\x65\x64\x65rationUserAccountId\x12#\n\rfederation_id\x18\x02 \x01(\tR\x0c\x66\x65\x64\x65rationId\"\x99\x01\n\x11UserAccountStatus\x12<\n\x05state\x18\x01 \x01(\x0e\x32&.nebius.iam.v1.UserAccountStatus.StateR\x05state\"F\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x01\x12\x0c\n\x08INACTIVE\x10\x02\x12\x0c\n\x08\x44\x45LETING\x10\x03\x42W\n\x14\x61i.nebius.pub.iam.v1B\x10UserAccountProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.user_account_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\020UserAccountProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _USERACCOUNTEXTERNALID.fields_by_name['federation_user_account_id']._options = None
  _USERACCOUNTEXTERNALID.fields_by_name['federation_user_account_id']._serialized_options = b'\300J\001'
  _globals['_USERACCOUNTEXTERNALID']._serialized_start=77
  _globals['_USERACCOUNTEXTERNALID']._serialized_end=203
  _globals['_USERACCOUNTSTATUS']._serialized_start=206
  _globals['_USERACCOUNTSTATUS']._serialized_end=359
  _globals['_USERACCOUNTSTATUS_STATE']._serialized_start=289
  _globals['_USERACCOUNTSTATUS_STATE']._serialized_end=359
# @@protoc_insertion_point(module_scope)
